#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Visualization
- Name     : Grids
- ID       : grid_visualisation

Description
----------
Visualization tools for grids.
'''

from PySAGA.helper import Tool_Wrapper

def Color_Palette_Rotation(
    GRID=None,
    COLORS=None,
    DOWN=None, Verbose=2):
    '''
    Color Palette Rotation
    ----------
    [grid_visualisation.0]\n
    The 'Color Palette Rotator' rotates the grids color palette.\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - COLORS [`colors`] : Colors
    - DOWN [`boolean`] : Down. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '0', 'Color Palette Rotation')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Option('COLORS', COLORS)
        Tool.Set_Option('DOWN', DOWN)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_0(
    GRID=None,
    COLORS=None,
    DOWN=None, Verbose=2):
    '''
    Color Palette Rotation
    ----------
    [grid_visualisation.0]\n
    The 'Color Palette Rotator' rotates the grids color palette.\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - COLORS [`colors`] : Colors
    - DOWN [`boolean`] : Down. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '0', 'Color Palette Rotation')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Option('COLORS', COLORS)
        Tool.Set_Option('DOWN', DOWN)
        return Tool.Execute(Verbose)
    return False

def Grid_Animation(
    GRIDS=None,
    GRID=None,
    FILE=None,
    FILE_NODATA=None,
    FILE_BGCOL=None,
    FILE_DELAY=None,
    FILE_COLORS=None,
    COLORS=None,
    NSTEPS=None,
    PROGRESS=None,
    LOOP=None,
    RANGE=None,
    RANGE_PERCENT=None,
    RANGE_STDDEV=None,
    RANGE_KEEP=None,
    RANGE_USER=None, Verbose=2):
    '''
    Grid Animation
    ----------
    [grid_visualisation.1]\n
    Creates an animation based on the values of selected grids. Previously known as 'Color Blending'.\n
    Arguments
    ----------
    - GRIDS [`input grid list`] : Grids
    - GRID [`output grid`] : Grid Animation
    - FILE [`file path`] : Save Frames. Store each frame as image file. If GIF format is selected one animated image file will be created.
    - FILE_NODATA [`boolean`] : Set Transparency for No-Data. Default: 1
    - FILE_BGCOL [`color`] : Background Color. Default: 16777215 Background color used for no-data cells when storing frames to file.
    - FILE_DELAY [`integer number`] : Delay. Minimum: 0 Default: 100 Delay, in milliseconds, to wait between each frame. Applies to animated GIF files.
    - FILE_COLORS [`integer number`] : Color Depth. Minimum: 2 Maximum: 256 Default: 236 Number of color entries used when storing animated GIF.
    - COLORS [`colors`] : Colors
    - NSTEPS [`integer number`] : Interpolation Steps. Minimum: 0 Default: 0
    - PROGRESS [`boolean`] : Progress Bar. Default: 0
    - LOOP [`choice`] : Loop. Available Choices: [0] do not loop [1] loop to first grid [2] loop Default: 0 Endless loop (3rd option) is ignored if file output is activated.
    - RANGE [`choice`] : Histogram Stretch. Available Choices: [0] each grid's range [1] each grid's standard deviation [2] overall range [3] overall standard deviation [4] user defined Default: 3
    - RANGE_PERCENT [`floating point number`] : Percent Stretch. Minimum: 0.000000 Maximum: 50.000000 Default: 2.000000
    - RANGE_STDDEV [`floating point number`] : Standard Deviation. Minimum: 0.000000 Default: 2.000000
    - RANGE_KEEP [`boolean`] : Keep in Range. Default: 1
    - RANGE_USER [`value range`] : Range

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '1', 'Grid Animation')
    if Tool.is_Okay():
        Tool.Set_Input ('GRIDS', GRIDS)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('FILE_NODATA', FILE_NODATA)
        Tool.Set_Option('FILE_BGCOL', FILE_BGCOL)
        Tool.Set_Option('FILE_DELAY', FILE_DELAY)
        Tool.Set_Option('FILE_COLORS', FILE_COLORS)
        Tool.Set_Option('COLORS', COLORS)
        Tool.Set_Option('NSTEPS', NSTEPS)
        Tool.Set_Option('PROGRESS', PROGRESS)
        Tool.Set_Option('LOOP', LOOP)
        Tool.Set_Option('RANGE', RANGE)
        Tool.Set_Option('RANGE_PERCENT', RANGE_PERCENT)
        Tool.Set_Option('RANGE_STDDEV', RANGE_STDDEV)
        Tool.Set_Option('RANGE_KEEP', RANGE_KEEP)
        Tool.Set_Option('RANGE_USER', RANGE_USER)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_1(
    GRIDS=None,
    GRID=None,
    FILE=None,
    FILE_NODATA=None,
    FILE_BGCOL=None,
    FILE_DELAY=None,
    FILE_COLORS=None,
    COLORS=None,
    NSTEPS=None,
    PROGRESS=None,
    LOOP=None,
    RANGE=None,
    RANGE_PERCENT=None,
    RANGE_STDDEV=None,
    RANGE_KEEP=None,
    RANGE_USER=None, Verbose=2):
    '''
    Grid Animation
    ----------
    [grid_visualisation.1]\n
    Creates an animation based on the values of selected grids. Previously known as 'Color Blending'.\n
    Arguments
    ----------
    - GRIDS [`input grid list`] : Grids
    - GRID [`output grid`] : Grid Animation
    - FILE [`file path`] : Save Frames. Store each frame as image file. If GIF format is selected one animated image file will be created.
    - FILE_NODATA [`boolean`] : Set Transparency for No-Data. Default: 1
    - FILE_BGCOL [`color`] : Background Color. Default: 16777215 Background color used for no-data cells when storing frames to file.
    - FILE_DELAY [`integer number`] : Delay. Minimum: 0 Default: 100 Delay, in milliseconds, to wait between each frame. Applies to animated GIF files.
    - FILE_COLORS [`integer number`] : Color Depth. Minimum: 2 Maximum: 256 Default: 236 Number of color entries used when storing animated GIF.
    - COLORS [`colors`] : Colors
    - NSTEPS [`integer number`] : Interpolation Steps. Minimum: 0 Default: 0
    - PROGRESS [`boolean`] : Progress Bar. Default: 0
    - LOOP [`choice`] : Loop. Available Choices: [0] do not loop [1] loop to first grid [2] loop Default: 0 Endless loop (3rd option) is ignored if file output is activated.
    - RANGE [`choice`] : Histogram Stretch. Available Choices: [0] each grid's range [1] each grid's standard deviation [2] overall range [3] overall standard deviation [4] user defined Default: 3
    - RANGE_PERCENT [`floating point number`] : Percent Stretch. Minimum: 0.000000 Maximum: 50.000000 Default: 2.000000
    - RANGE_STDDEV [`floating point number`] : Standard Deviation. Minimum: 0.000000 Default: 2.000000
    - RANGE_KEEP [`boolean`] : Keep in Range. Default: 1
    - RANGE_USER [`value range`] : Range

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '1', 'Grid Animation')
    if Tool.is_Okay():
        Tool.Set_Input ('GRIDS', GRIDS)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('FILE_NODATA', FILE_NODATA)
        Tool.Set_Option('FILE_BGCOL', FILE_BGCOL)
        Tool.Set_Option('FILE_DELAY', FILE_DELAY)
        Tool.Set_Option('FILE_COLORS', FILE_COLORS)
        Tool.Set_Option('COLORS', COLORS)
        Tool.Set_Option('NSTEPS', NSTEPS)
        Tool.Set_Option('PROGRESS', PROGRESS)
        Tool.Set_Option('LOOP', LOOP)
        Tool.Set_Option('RANGE', RANGE)
        Tool.Set_Option('RANGE_PERCENT', RANGE_PERCENT)
        Tool.Set_Option('RANGE_STDDEV', RANGE_STDDEV)
        Tool.Set_Option('RANGE_KEEP', RANGE_KEEP)
        Tool.Set_Option('RANGE_USER', RANGE_USER)
        return Tool.Execute(Verbose)
    return False

def Fit_Color_Palette_to_Grid_Values(
    GRID=None,
    COUNT=None,
    SCALE=None,
    RANGE=None, Verbose=2):
    '''
    Fit Color Palette to Grid Values
    ----------
    [grid_visualisation.2]\n
    Fit Color Palette to Grid Values\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - COUNT [`integer number`] : Number of Colors. Minimum: 2 Default: 100
    - SCALE [`choice`] : Scale. Available Choices: [0] Grid range [1] User defined range Default: 0
    - RANGE [`value range`] : User defined range

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '2', 'Fit Color Palette to Grid Values')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Option('COUNT', COUNT)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('RANGE', RANGE)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_2(
    GRID=None,
    COUNT=None,
    SCALE=None,
    RANGE=None, Verbose=2):
    '''
    Fit Color Palette to Grid Values
    ----------
    [grid_visualisation.2]\n
    Fit Color Palette to Grid Values\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - COUNT [`integer number`] : Number of Colors. Minimum: 2 Default: 100
    - SCALE [`choice`] : Scale. Available Choices: [0] Grid range [1] User defined range Default: 0
    - RANGE [`value range`] : User defined range

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '2', 'Fit Color Palette to Grid Values')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Option('COUNT', COUNT)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('RANGE', RANGE)
        return Tool.Execute(Verbose)
    return False

def RGB_Composite(
    R_GRID=None,
    G_GRID=None,
    B_GRID=None,
    A_GRID=None,
    RGB=None,
    METHOD=None,
    RANGE=None,
    PERCTL=None,
    STDDEV=None,
    NODATA=None, Verbose=2):
    '''
    RGB Composite
    ----------
    [grid_visualisation.3]\n
    Create red-green-blue overlays of grids.\n
    Arguments
    ----------
    - R_GRID [`input grid`] : Red
    - G_GRID [`input grid`] : Green
    - B_GRID [`input grid`] : Blue
    - A_GRID [`optional input grid`] : Alpha
    - RGB [`output grid`] : Composite
    - METHOD [`choice`] : Value Preparation. Available Choices: [0] take original value (0 - 255) [1] rescale to 0 - 255 [2] user defined [3] percentiles [4] standard deviation Default: 0
    - RANGE [`value range`] : Rescale Range
    - PERCTL [`value range`] : Percentiles
    - STDDEV [`floating point number`] : Standard Deviation. Minimum: 0.000000 Default: 2.000000
    - NODATA [`boolean`] : Ignore No Data Cells. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '3', 'RGB Composite')
    if Tool.is_Okay():
        Tool.Set_Input ('R_GRID', R_GRID)
        Tool.Set_Input ('G_GRID', G_GRID)
        Tool.Set_Input ('B_GRID', B_GRID)
        Tool.Set_Input ('A_GRID', A_GRID)
        Tool.Set_Output('RGB', RGB)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('RANGE', RANGE)
        Tool.Set_Option('PERCTL', PERCTL)
        Tool.Set_Option('STDDEV', STDDEV)
        Tool.Set_Option('NODATA', NODATA)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_3(
    R_GRID=None,
    G_GRID=None,
    B_GRID=None,
    A_GRID=None,
    RGB=None,
    METHOD=None,
    RANGE=None,
    PERCTL=None,
    STDDEV=None,
    NODATA=None, Verbose=2):
    '''
    RGB Composite
    ----------
    [grid_visualisation.3]\n
    Create red-green-blue overlays of grids.\n
    Arguments
    ----------
    - R_GRID [`input grid`] : Red
    - G_GRID [`input grid`] : Green
    - B_GRID [`input grid`] : Blue
    - A_GRID [`optional input grid`] : Alpha
    - RGB [`output grid`] : Composite
    - METHOD [`choice`] : Value Preparation. Available Choices: [0] take original value (0 - 255) [1] rescale to 0 - 255 [2] user defined [3] percentiles [4] standard deviation Default: 0
    - RANGE [`value range`] : Rescale Range
    - PERCTL [`value range`] : Percentiles
    - STDDEV [`floating point number`] : Standard Deviation. Minimum: 0.000000 Default: 2.000000
    - NODATA [`boolean`] : Ignore No Data Cells. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '3', 'RGB Composite')
    if Tool.is_Okay():
        Tool.Set_Input ('R_GRID', R_GRID)
        Tool.Set_Input ('G_GRID', G_GRID)
        Tool.Set_Input ('B_GRID', B_GRID)
        Tool.Set_Input ('A_GRID', A_GRID)
        Tool.Set_Output('RGB', RGB)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('RANGE', RANGE)
        Tool.Set_Option('PERCTL', PERCTL)
        Tool.Set_Option('STDDEV', STDDEV)
        Tool.Set_Option('NODATA', NODATA)
        return Tool.Execute(Verbose)
    return False

def Create_3D_Image(
    DEM=None,
    IMAGE=None,
    SHAPES=None,
    RGB=None,
    RGB_Z=None,
    ZEXAGG=None,
    ZEXAGG_MIN=None,
    Z_ROTATE=None,
    X_ROTATE=None,
    X_ROTATE_LEVEL=None,
    PANBREAK=None,
    BKCOLOR=None,
    PROJECTION=None,
    NX=None,
    NY=None, Verbose=2):
    '''
    Create 3D Image
    ----------
    [grid_visualisation.4]\n
    Create 3D Image\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - IMAGE [`input grid`] : Overlay Image
    - SHAPES [`optional input shapes list`] : Shapes to project
    - RGB [`output grid`] : 3D Image
    - RGB_Z [`output grid`] : Projected Height
    - ZEXAGG [`floating point number`] : Exaggeration. Default: 1.000000
    - ZEXAGG_MIN [`floating point number`] : Minimum Exaggeration [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 10.000000
    - Z_ROTATE [`floating point number`] : Image Rotation [Degree]. Default: 0.000000
    - X_ROTATE [`floating point number`] : Local Rotation [Degree]. Default: 1.000000
    - X_ROTATE_LEVEL [`choice`] : Local Rotation Base Level. Available Choices: [0] Zero [1] Mean Elevation Default: 1
    - PANBREAK [`floating point number`] : Panorama Break [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 70.000000
    - BKCOLOR [`color`] : Background Color. Default: 0
    - PROJECTION [`choice`] : Projection. Available Choices: [0] Panorama [1] Circular Default: 0
    - NX [`integer number`] : 3D Image Width. Minimum: 1 Default: 100
    - NY [`integer number`] : 3D Image Height. Minimum: 1 Default: 100

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '4', 'Create 3D Image')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('IMAGE', IMAGE)
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Output('RGB', RGB)
        Tool.Set_Output('RGB_Z', RGB_Z)
        Tool.Set_Option('ZEXAGG', ZEXAGG)
        Tool.Set_Option('ZEXAGG_MIN', ZEXAGG_MIN)
        Tool.Set_Option('Z_ROTATE', Z_ROTATE)
        Tool.Set_Option('X_ROTATE', X_ROTATE)
        Tool.Set_Option('X_ROTATE_LEVEL', X_ROTATE_LEVEL)
        Tool.Set_Option('PANBREAK', PANBREAK)
        Tool.Set_Option('BKCOLOR', BKCOLOR)
        Tool.Set_Option('PROJECTION', PROJECTION)
        Tool.Set_Option('NX', NX)
        Tool.Set_Option('NY', NY)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_4(
    DEM=None,
    IMAGE=None,
    SHAPES=None,
    RGB=None,
    RGB_Z=None,
    ZEXAGG=None,
    ZEXAGG_MIN=None,
    Z_ROTATE=None,
    X_ROTATE=None,
    X_ROTATE_LEVEL=None,
    PANBREAK=None,
    BKCOLOR=None,
    PROJECTION=None,
    NX=None,
    NY=None, Verbose=2):
    '''
    Create 3D Image
    ----------
    [grid_visualisation.4]\n
    Create 3D Image\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - IMAGE [`input grid`] : Overlay Image
    - SHAPES [`optional input shapes list`] : Shapes to project
    - RGB [`output grid`] : 3D Image
    - RGB_Z [`output grid`] : Projected Height
    - ZEXAGG [`floating point number`] : Exaggeration. Default: 1.000000
    - ZEXAGG_MIN [`floating point number`] : Minimum Exaggeration [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 10.000000
    - Z_ROTATE [`floating point number`] : Image Rotation [Degree]. Default: 0.000000
    - X_ROTATE [`floating point number`] : Local Rotation [Degree]. Default: 1.000000
    - X_ROTATE_LEVEL [`choice`] : Local Rotation Base Level. Available Choices: [0] Zero [1] Mean Elevation Default: 1
    - PANBREAK [`floating point number`] : Panorama Break [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 70.000000
    - BKCOLOR [`color`] : Background Color. Default: 0
    - PROJECTION [`choice`] : Projection. Available Choices: [0] Panorama [1] Circular Default: 0
    - NX [`integer number`] : 3D Image Width. Minimum: 1 Default: 100
    - NY [`integer number`] : 3D Image Height. Minimum: 1 Default: 100

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '4', 'Create 3D Image')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('IMAGE', IMAGE)
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Output('RGB', RGB)
        Tool.Set_Output('RGB_Z', RGB_Z)
        Tool.Set_Option('ZEXAGG', ZEXAGG)
        Tool.Set_Option('ZEXAGG_MIN', ZEXAGG_MIN)
        Tool.Set_Option('Z_ROTATE', Z_ROTATE)
        Tool.Set_Option('X_ROTATE', X_ROTATE)
        Tool.Set_Option('X_ROTATE_LEVEL', X_ROTATE_LEVEL)
        Tool.Set_Option('PANBREAK', PANBREAK)
        Tool.Set_Option('BKCOLOR', BKCOLOR)
        Tool.Set_Option('PROJECTION', PROJECTION)
        Tool.Set_Option('NX', NX)
        Tool.Set_Option('NY', NY)
        return Tool.Execute(Verbose)
    return False

def Color_Triangle_Composite(
    A_GRID=None,
    B_GRID=None,
    C_GRID=None,
    GRID=None,
    A_COLOR=None,
    A_METHOD=None,
    A_RANGE=None,
    A_PERCTL=None,
    A_PERCENT=None,
    B_COLOR=None,
    B_METHOD=None,
    B_RANGE=None,
    B_PERCTL=None,
    B_PERCENT=None,
    C_COLOR=None,
    C_METHOD=None,
    C_RANGE=None,
    C_PERCTL=None,
    C_PERCENT=None, Verbose=2):
    '''
    Color Triangle Composite
    ----------
    [grid_visualisation.5]\n
    Similar to 'RGB Composite', but the three colors representing intensity of each data set can be chosen by user.\n
    Arguments
    ----------
    - A_GRID [`input grid`] : A
    - B_GRID [`input grid`] : B
    - C_GRID [`input grid`] : C
    - GRID [`output grid`] : Composite
    - A_COLOR [`color`] : Color. Default: 255
    - A_METHOD [`choice`] : Value Preparation. Available Choices: [0] 0 - 1 [1] Rescale to 0 - 1 [2] User defined rescale [3] Percentiles [4] Percentage of standard deviation Default: 4
    - A_RANGE [`value range`] : Rescale Range
    - A_PERCTL [`value range`] : Percentiles
    - A_PERCENT [`floating point number`] : Percentage of Standard Deviation. Minimum: 0.000000 Default: 150.000000
    - B_COLOR [`color`] : Color. Default: 65280
    - B_METHOD [`choice`] : Value Preparation. Available Choices: [0] 0 - 1 [1] Rescale to 0 - 1 [2] User defined rescale [3] Percentiles [4] Percentage of standard deviation Default: 4
    - B_RANGE [`value range`] : Rescale Range
    - B_PERCTL [`value range`] : Percentiles
    - B_PERCENT [`floating point number`] : Percentage of standard deviation. Minimum: 0.000000 Default: 150.000000
    - C_COLOR [`color`] : Color. Default: 16711680
    - C_METHOD [`choice`] : Value Preparation. Available Choices: [0] 0 - 1 [1] Rescale to 0 - 1 [2] User defined rescale [3] Percentiles [4] Percentage of standard deviation Default: 4
    - C_RANGE [`value range`] : Rescale Range
    - C_PERCTL [`value range`] : Percentiles
    - C_PERCENT [`floating point number`] : Percentage of Standard Deviation. Minimum: 0.000000 Default: 150.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '5', 'Color Triangle Composite')
    if Tool.is_Okay():
        Tool.Set_Input ('A_GRID', A_GRID)
        Tool.Set_Input ('B_GRID', B_GRID)
        Tool.Set_Input ('C_GRID', C_GRID)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Option('A_COLOR', A_COLOR)
        Tool.Set_Option('A_METHOD', A_METHOD)
        Tool.Set_Option('A_RANGE', A_RANGE)
        Tool.Set_Option('A_PERCTL', A_PERCTL)
        Tool.Set_Option('A_PERCENT', A_PERCENT)
        Tool.Set_Option('B_COLOR', B_COLOR)
        Tool.Set_Option('B_METHOD', B_METHOD)
        Tool.Set_Option('B_RANGE', B_RANGE)
        Tool.Set_Option('B_PERCTL', B_PERCTL)
        Tool.Set_Option('B_PERCENT', B_PERCENT)
        Tool.Set_Option('C_COLOR', C_COLOR)
        Tool.Set_Option('C_METHOD', C_METHOD)
        Tool.Set_Option('C_RANGE', C_RANGE)
        Tool.Set_Option('C_PERCTL', C_PERCTL)
        Tool.Set_Option('C_PERCENT', C_PERCENT)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_5(
    A_GRID=None,
    B_GRID=None,
    C_GRID=None,
    GRID=None,
    A_COLOR=None,
    A_METHOD=None,
    A_RANGE=None,
    A_PERCTL=None,
    A_PERCENT=None,
    B_COLOR=None,
    B_METHOD=None,
    B_RANGE=None,
    B_PERCTL=None,
    B_PERCENT=None,
    C_COLOR=None,
    C_METHOD=None,
    C_RANGE=None,
    C_PERCTL=None,
    C_PERCENT=None, Verbose=2):
    '''
    Color Triangle Composite
    ----------
    [grid_visualisation.5]\n
    Similar to 'RGB Composite', but the three colors representing intensity of each data set can be chosen by user.\n
    Arguments
    ----------
    - A_GRID [`input grid`] : A
    - B_GRID [`input grid`] : B
    - C_GRID [`input grid`] : C
    - GRID [`output grid`] : Composite
    - A_COLOR [`color`] : Color. Default: 255
    - A_METHOD [`choice`] : Value Preparation. Available Choices: [0] 0 - 1 [1] Rescale to 0 - 1 [2] User defined rescale [3] Percentiles [4] Percentage of standard deviation Default: 4
    - A_RANGE [`value range`] : Rescale Range
    - A_PERCTL [`value range`] : Percentiles
    - A_PERCENT [`floating point number`] : Percentage of Standard Deviation. Minimum: 0.000000 Default: 150.000000
    - B_COLOR [`color`] : Color. Default: 65280
    - B_METHOD [`choice`] : Value Preparation. Available Choices: [0] 0 - 1 [1] Rescale to 0 - 1 [2] User defined rescale [3] Percentiles [4] Percentage of standard deviation Default: 4
    - B_RANGE [`value range`] : Rescale Range
    - B_PERCTL [`value range`] : Percentiles
    - B_PERCENT [`floating point number`] : Percentage of standard deviation. Minimum: 0.000000 Default: 150.000000
    - C_COLOR [`color`] : Color. Default: 16711680
    - C_METHOD [`choice`] : Value Preparation. Available Choices: [0] 0 - 1 [1] Rescale to 0 - 1 [2] User defined rescale [3] Percentiles [4] Percentage of standard deviation Default: 4
    - C_RANGE [`value range`] : Rescale Range
    - C_PERCTL [`value range`] : Percentiles
    - C_PERCENT [`floating point number`] : Percentage of Standard Deviation. Minimum: 0.000000 Default: 150.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '5', 'Color Triangle Composite')
    if Tool.is_Okay():
        Tool.Set_Input ('A_GRID', A_GRID)
        Tool.Set_Input ('B_GRID', B_GRID)
        Tool.Set_Input ('C_GRID', C_GRID)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Option('A_COLOR', A_COLOR)
        Tool.Set_Option('A_METHOD', A_METHOD)
        Tool.Set_Option('A_RANGE', A_RANGE)
        Tool.Set_Option('A_PERCTL', A_PERCTL)
        Tool.Set_Option('A_PERCENT', A_PERCENT)
        Tool.Set_Option('B_COLOR', B_COLOR)
        Tool.Set_Option('B_METHOD', B_METHOD)
        Tool.Set_Option('B_RANGE', B_RANGE)
        Tool.Set_Option('B_PERCTL', B_PERCTL)
        Tool.Set_Option('B_PERCENT', B_PERCENT)
        Tool.Set_Option('C_COLOR', C_COLOR)
        Tool.Set_Option('C_METHOD', C_METHOD)
        Tool.Set_Option('C_RANGE', C_RANGE)
        Tool.Set_Option('C_PERCTL', C_PERCTL)
        Tool.Set_Option('C_PERCENT', C_PERCENT)
        return Tool.Execute(Verbose)
    return False

def Histogram_Surface(
    GRID=None,
    HIST=None,
    METHOD=None, Verbose=2):
    '''
    Histogram Surface
    ----------
    [grid_visualisation.6]\n
    Histogram Surface\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - HIST [`output data object`] : Histogram
    - METHOD [`choice`] : Method. Available Choices: [0] rows [1] columns [2] circle Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '6', 'Histogram Surface')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Output('HIST', HIST)
        Tool.Set_Option('METHOD', METHOD)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_6(
    GRID=None,
    HIST=None,
    METHOD=None, Verbose=2):
    '''
    Histogram Surface
    ----------
    [grid_visualisation.6]\n
    Histogram Surface\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - HIST [`output data object`] : Histogram
    - METHOD [`choice`] : Method. Available Choices: [0] rows [1] columns [2] circle Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '6', 'Histogram Surface')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Output('HIST', HIST)
        Tool.Set_Option('METHOD', METHOD)
        return Tool.Execute(Verbose)
    return False

def AspectSlope_Grid(
    ASPECT=None,
    SLOPE=None,
    ASPECT_SLOPE=None,
    LUT=None, Verbose=2):
    '''
    Aspect-Slope Grid
    ----------
    [grid_visualisation.7]\n
    This tool creates an aspect-slope map which shows both the aspect and the slope of the terrain. Aspect is symbolized by different hues, while slope is mapped with saturation.\n
    References:\n
    Brewer, C.A. & Marlow, K.A. (1993): Color Representation of Aspect and Slope simultaneously. Proceedings, Eleventh International Symposium on Computer-Assisted Cartography (Auto-Carto-11), Minneapolis, October/November 1993, pp. 328-337.\n
    [http://www.personal.psu.edu/cab38/Terrain/AutoCarto.html](http://www.personal.psu.edu/cab38/Terrain/AutoCarto.html)\n
    [http://blogs.esri.com/esri/arcgis/2008/05/23/aspect-slope-map/](http://blogs.esri.com/esri/arcgis/2008/05/23/aspect-slope-map/)\n
    Arguments
    ----------
    - ASPECT [`input grid`] : Aspect. Aspect grid, in radians and 360 degree from north.
    - SLOPE [`input grid`] : Slope. Slope grid, in radians.
    - ASPECT_SLOPE [`output grid`] : Aspect-Slope. Final aspect-slope grid.
    - LUT [`output table`] : Lookup Table. Lookup table.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '7', 'Aspect-Slope Grid')
    if Tool.is_Okay():
        Tool.Set_Input ('ASPECT', ASPECT)
        Tool.Set_Input ('SLOPE', SLOPE)
        Tool.Set_Output('ASPECT_SLOPE', ASPECT_SLOPE)
        Tool.Set_Output('LUT', LUT)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_7(
    ASPECT=None,
    SLOPE=None,
    ASPECT_SLOPE=None,
    LUT=None, Verbose=2):
    '''
    Aspect-Slope Grid
    ----------
    [grid_visualisation.7]\n
    This tool creates an aspect-slope map which shows both the aspect and the slope of the terrain. Aspect is symbolized by different hues, while slope is mapped with saturation.\n
    References:\n
    Brewer, C.A. & Marlow, K.A. (1993): Color Representation of Aspect and Slope simultaneously. Proceedings, Eleventh International Symposium on Computer-Assisted Cartography (Auto-Carto-11), Minneapolis, October/November 1993, pp. 328-337.\n
    [http://www.personal.psu.edu/cab38/Terrain/AutoCarto.html](http://www.personal.psu.edu/cab38/Terrain/AutoCarto.html)\n
    [http://blogs.esri.com/esri/arcgis/2008/05/23/aspect-slope-map/](http://blogs.esri.com/esri/arcgis/2008/05/23/aspect-slope-map/)\n
    Arguments
    ----------
    - ASPECT [`input grid`] : Aspect. Aspect grid, in radians and 360 degree from north.
    - SLOPE [`input grid`] : Slope. Slope grid, in radians.
    - ASPECT_SLOPE [`output grid`] : Aspect-Slope. Final aspect-slope grid.
    - LUT [`output table`] : Lookup Table. Lookup table.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '7', 'Aspect-Slope Grid')
    if Tool.is_Okay():
        Tool.Set_Input ('ASPECT', ASPECT)
        Tool.Set_Input ('SLOPE', SLOPE)
        Tool.Set_Output('ASPECT_SLOPE', ASPECT_SLOPE)
        Tool.Set_Output('LUT', LUT)
        return Tool.Execute(Verbose)
    return False

def Terrain_Map_View(
    DEM=None,
    SHADE=None,
    OPENNESS=None,
    SLOPE=None,
    CONTOURS=None,
    METHOD=None,
    RADIUS=None,
    CONTOUR_LINES=None,
    EQUIDISTANCE=None, Verbose=2):
    '''
    Terrain Map View
    ----------
    [grid_visualisation.8]\n
    This tool allows one to create different terrain visualisations from an elevation dataset:\n
    * Topography: a simple map with an analytical hillshading of the terrain\n
    * Morphology: a map which visualizes the terrain by combining positive and negative openness (Yokoyama et al. 2002) with terrain slope in a single map. In contrast to conventional shading methods this has the advantage of being independent from the direction of the light source.\n
    Arguments
    ----------
    - DEM [`input grid`] : DEM. Digital elevation model.
    - SHADE [`output grid`] : Shade. The shaded DTM.
    - OPENNESS [`output grid`] : Openness. The difference of positive and negative openness.
    - SLOPE [`output grid`] : Slope. The calculated terrain slope [radians].
    - CONTOURS [`output shapes`] : Contours. The generated contour lines.
    - METHOD [`choice`] : Method. Available Choices: [0] Topography [1] Morphology Default: 0 Choose the map type to generate.
    - RADIUS [`floating point number`] : Radial Limit. Minimum: 0.000000 Default: 1000.000000 Radial search limit for openness calculation.
    - CONTOUR_LINES [`boolean`] : Contour Lines. Default: 1 Derive contour lines.
    - EQUIDISTANCE [`floating point number`] : Equidistance. Minimum: 0.000000 Default: 50.000000 Contour lines equidistance [map units].

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '8', 'Terrain Map View')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Output('SHADE', SHADE)
        Tool.Set_Output('OPENNESS', OPENNESS)
        Tool.Set_Output('SLOPE', SLOPE)
        Tool.Set_Output('CONTOURS', CONTOURS)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('RADIUS', RADIUS)
        Tool.Set_Option('CONTOUR_LINES', CONTOUR_LINES)
        Tool.Set_Option('EQUIDISTANCE', EQUIDISTANCE)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_8(
    DEM=None,
    SHADE=None,
    OPENNESS=None,
    SLOPE=None,
    CONTOURS=None,
    METHOD=None,
    RADIUS=None,
    CONTOUR_LINES=None,
    EQUIDISTANCE=None, Verbose=2):
    '''
    Terrain Map View
    ----------
    [grid_visualisation.8]\n
    This tool allows one to create different terrain visualisations from an elevation dataset:\n
    * Topography: a simple map with an analytical hillshading of the terrain\n
    * Morphology: a map which visualizes the terrain by combining positive and negative openness (Yokoyama et al. 2002) with terrain slope in a single map. In contrast to conventional shading methods this has the advantage of being independent from the direction of the light source.\n
    Arguments
    ----------
    - DEM [`input grid`] : DEM. Digital elevation model.
    - SHADE [`output grid`] : Shade. The shaded DTM.
    - OPENNESS [`output grid`] : Openness. The difference of positive and negative openness.
    - SLOPE [`output grid`] : Slope. The calculated terrain slope [radians].
    - CONTOURS [`output shapes`] : Contours. The generated contour lines.
    - METHOD [`choice`] : Method. Available Choices: [0] Topography [1] Morphology Default: 0 Choose the map type to generate.
    - RADIUS [`floating point number`] : Radial Limit. Minimum: 0.000000 Default: 1000.000000 Radial search limit for openness calculation.
    - CONTOUR_LINES [`boolean`] : Contour Lines. Default: 1 Derive contour lines.
    - EQUIDISTANCE [`floating point number`] : Equidistance. Minimum: 0.000000 Default: 50.000000 Contour lines equidistance [map units].

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '8', 'Terrain Map View')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Output('SHADE', SHADE)
        Tool.Set_Output('OPENNESS', OPENNESS)
        Tool.Set_Output('SLOPE', SLOPE)
        Tool.Set_Output('CONTOURS', CONTOURS)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('RADIUS', RADIUS)
        Tool.Set_Option('CONTOUR_LINES', CONTOUR_LINES)
        Tool.Set_Option('EQUIDISTANCE', EQUIDISTANCE)
        return Tool.Execute(Verbose)
    return False

def Split_RGB_Composite(
    RGB=None,
    R=None,
    G=None,
    B=None,
    A=None,
    NODATA=None, Verbose=2):
    '''
    Split RGB Composite
    ----------
    [grid_visualisation.9]\n
    Split red-green-blue channels of an rgb coded grid.\n
    Arguments
    ----------
    - RGB [`input grid`] : RGB Composite
    - R [`output grid`] : Red
    - G [`output grid`] : Green
    - B [`output grid`] : Blue
    - A [`output grid`] : Alpha
    - NODATA [`boolean`] : Ignore No Data. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '9', 'Split RGB Composite')
    if Tool.is_Okay():
        Tool.Set_Input ('RGB', RGB)
        Tool.Set_Output('R', R)
        Tool.Set_Output('G', G)
        Tool.Set_Output('B', B)
        Tool.Set_Output('A', A)
        Tool.Set_Option('NODATA', NODATA)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_9(
    RGB=None,
    R=None,
    G=None,
    B=None,
    A=None,
    NODATA=None, Verbose=2):
    '''
    Split RGB Composite
    ----------
    [grid_visualisation.9]\n
    Split red-green-blue channels of an rgb coded grid.\n
    Arguments
    ----------
    - RGB [`input grid`] : RGB Composite
    - R [`output grid`] : Red
    - G [`output grid`] : Green
    - B [`output grid`] : Blue
    - A [`output grid`] : Alpha
    - NODATA [`boolean`] : Ignore No Data. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '9', 'Split RGB Composite')
    if Tool.is_Okay():
        Tool.Set_Input ('RGB', RGB)
        Tool.Set_Output('R', R)
        Tool.Set_Output('G', G)
        Tool.Set_Output('B', B)
        Tool.Set_Output('A', A)
        Tool.Set_Option('NODATA', NODATA)
        return Tool.Execute(Verbose)
    return False

def Select_Lookup_Table_for_Grid_Visualization(
    GRID=None,
    LUT=None,
    NAME=None,
    VALUE=None,
    VALUE_MAX=None,
    DESCRIPTION=None,
    COLOR=None, Verbose=2):
    '''
    Select Look-up Table for Grid Visualization
    ----------
    [grid_visualisation.10]\n
    Select a look-up table for visual classification of a grid. Useful in combination with tool chains.\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - LUT [`input table`] : Look-up Table
    - NAME [`table field`] : Name
    - VALUE [`table field`] : Value
    - VALUE_MAX [`table field`] : Value (Range Maximum)
    - DESCRIPTION [`table field`] : Description
    - COLOR [`table field`] : Color

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '10', 'Select Look-up Table for Grid Visualization')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Input ('LUT', LUT)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('VALUE', VALUE)
        Tool.Set_Option('VALUE_MAX', VALUE_MAX)
        Tool.Set_Option('DESCRIPTION', DESCRIPTION)
        Tool.Set_Option('COLOR', COLOR)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_10(
    GRID=None,
    LUT=None,
    NAME=None,
    VALUE=None,
    VALUE_MAX=None,
    DESCRIPTION=None,
    COLOR=None, Verbose=2):
    '''
    Select Look-up Table for Grid Visualization
    ----------
    [grid_visualisation.10]\n
    Select a look-up table for visual classification of a grid. Useful in combination with tool chains.\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - LUT [`input table`] : Look-up Table
    - NAME [`table field`] : Name
    - VALUE [`table field`] : Value
    - VALUE_MAX [`table field`] : Value (Range Maximum)
    - DESCRIPTION [`table field`] : Description
    - COLOR [`table field`] : Color

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '10', 'Select Look-up Table for Grid Visualization')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Input ('LUT', LUT)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('VALUE', VALUE)
        Tool.Set_Option('VALUE_MAX', VALUE_MAX)
        Tool.Set_Option('DESCRIPTION', DESCRIPTION)
        Tool.Set_Option('COLOR', COLOR)
        return Tool.Execute(Verbose)
    return False

def Create_a_Table_from_Lookup_Table(
    TABLE=None,
    LUT=None, Verbose=2):
    '''
    Create a Table from Look-up Table
    ----------
    [grid_visualisation.11]\n
    Creates a table object from a look-up table for visual data object classifications. Useful in combination with tool chains.\n
    Arguments
    ----------
    - TABLE [`output table`] : Table
    - LUT [`static table`] : Look-up Table. 5 Fields: - 1. [color] COLOR - 2. [string] NAME - 3. [string] DESCRIPTION - 4. [8 byte floating point number] MINIMUM - 5. [8 byte floating point number] MAXIMUM 

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '11', 'Create a Table from Look-up Table')
    if Tool.is_Okay():
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('LUT', LUT)
        return Tool.Execute(Verbose)
    return False

def run_tool_grid_visualisation_11(
    TABLE=None,
    LUT=None, Verbose=2):
    '''
    Create a Table from Look-up Table
    ----------
    [grid_visualisation.11]\n
    Creates a table object from a look-up table for visual data object classifications. Useful in combination with tool chains.\n
    Arguments
    ----------
    - TABLE [`output table`] : Table
    - LUT [`static table`] : Look-up Table. 5 Fields: - 1. [color] COLOR - 2. [string] NAME - 3. [string] DESCRIPTION - 4. [8 byte floating point number] MINIMUM - 5. [8 byte floating point number] MAXIMUM 

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('grid_visualisation', '11', 'Create a Table from Look-up Table')
    if Tool.is_Okay():
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('LUT', LUT)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

#################################################################################
# MIT License

# Copyright (c) 2023 Olaf Conrad

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#################################################################################
# Purpose
#################################################################################

'''
The PySAGA.convert module provides a collection of conversion tools.
'''


#################################################################################
#
# Globals
#________________________________________________________________________________

from PySAGA import saga_api


#################################################################################
#
# numpy...
#________________________________________________________________________________

#################################################################################
#________________________________________________________________________________
def Table_To_NumPy(Table, yFields=[], xField=None):
    '''
    Converts a saga_api.CSG_Grid object into a list of numpy arrays.
    '''
    try:
        import numpy
    except:
        return None

    try:
        Table.Get_Count()
    except:
        return None

    if xField:
        try:
            xField = int(xField)
        except:
            xField = -1
    else:
        xField = -1

    Fields = []; Data = [numpy.empty(Table.Get_Count(), float)]

    for i in range(0, len(yFields)):
        try:
            yFields[i] = int(yFields[i])
            if yFields[i] >= 0 and yFields[i] < Table.Get_Field_Count():
                Fields.append(yFields[i]); Data.append(numpy.empty(Table.Get_Count()))
        except:
            yFields[i] = -1

    for i in range(0, Table.Get_Count()):
        Record = Table.Get_Record(i)
        Data[0][i] = i if xField < 0 else Record.asDouble(xField)
        for j in range(1, len(Data)):
            Data[j][i] = Record.asDouble(Fields[j - 1])

    return Data


#################################################################################
#________________________________________________________________________________
def Grid_To_NumPy(Grid):
    '''
    Converts a saga_api.CSG_Grid object to numpy arrays.
    '''
    try:
        import numpy
    except:
        return None

    X, Y = numpy.meshgrid(
        numpy.linspace(Grid.Get_XMin(), Grid.Get_XMax(), Grid.Get_NX()),
        numpy.linspace(Grid.Get_YMin(), Grid.Get_YMax(), Grid.Get_NY())
    )

    Z = X + Y

    for y in range(0, Grid.Get_NY()):
        yy = Grid.Get_NY() - 1 - y
        for x in range(0, Grid.Get_NX()):
            Z[y, x] = Grid.asDouble(x, yy)

    return X, Y, Z


#################################################################################
#
#________________________________________________________________________________



#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Import/Export
- Name     : Virtual
- ID       : io_virtual

Description
----------
Tools for the handling of virtual datasets.
'''

from PySAGA.helper import Tool_Wrapper

def Create_Virtual_Point_Cloud_Dataset(
    FILES=None,
    INPUT_FILE_LIST=None,
    FILENAME=None,
    METHOD_PATHS=None,
    USE_HEADER=None, Verbose=2):
    '''
    Create Virtual Point Cloud Dataset
    ----------
    [io_virtual.0]\n
    The tool allows one to create a virtual point cloud dataset from a set of SAGA point cloud files. For a large number of files, it is advised to use an input file list, i.e. a text file with the full path to an input point cloud on each line. If possible, you should make use of the point cloud headers files to construct the virtual dataset. This avoids that each dataset has to be loaded and thus reduces execution time enormously.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which describes a mosaic of individual point cloud files. Such a virtual point cloud dataset can be used for seamless data access with the 'Get Subset from Virtual Point Cloud' tool.\n
    All point cloud input datasets must share the same attribute table structure, NoData value and projection.\n
    Arguments
    ----------
    - FILES [`file path`] : Input Files. The input point cloud files to use
    - INPUT_FILE_LIST [`file path`] : Input File List. A text file with the full path to an input point cloud on each line
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file
    - METHOD_PATHS [`choice`] : File Paths. Available Choices: [0] absolute [1] relative Default: 1 Choose how to handle file paths. With relative paths, you can package the *.spcvf and your point cloud tiles easily.
    - USE_HEADER [`boolean`] : Use Header File. Default: 0 Check this parameter to use (only) the point cloud header file to construct the virtual dataset.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '0', 'Create Virtual Point Cloud Dataset')
    if Tool.is_Okay():
        Tool.Set_Option('FILES', FILES)
        Tool.Set_Option('INPUT_FILE_LIST', INPUT_FILE_LIST)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('METHOD_PATHS', METHOD_PATHS)
        Tool.Set_Option('USE_HEADER', USE_HEADER)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_virtual_0(
    FILES=None,
    INPUT_FILE_LIST=None,
    FILENAME=None,
    METHOD_PATHS=None,
    USE_HEADER=None, Verbose=2):
    '''
    Create Virtual Point Cloud Dataset
    ----------
    [io_virtual.0]\n
    The tool allows one to create a virtual point cloud dataset from a set of SAGA point cloud files. For a large number of files, it is advised to use an input file list, i.e. a text file with the full path to an input point cloud on each line. If possible, you should make use of the point cloud headers files to construct the virtual dataset. This avoids that each dataset has to be loaded and thus reduces execution time enormously.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which describes a mosaic of individual point cloud files. Such a virtual point cloud dataset can be used for seamless data access with the 'Get Subset from Virtual Point Cloud' tool.\n
    All point cloud input datasets must share the same attribute table structure, NoData value and projection.\n
    Arguments
    ----------
    - FILES [`file path`] : Input Files. The input point cloud files to use
    - INPUT_FILE_LIST [`file path`] : Input File List. A text file with the full path to an input point cloud on each line
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file
    - METHOD_PATHS [`choice`] : File Paths. Available Choices: [0] absolute [1] relative Default: 1 Choose how to handle file paths. With relative paths, you can package the *.spcvf and your point cloud tiles easily.
    - USE_HEADER [`boolean`] : Use Header File. Default: 0 Check this parameter to use (only) the point cloud header file to construct the virtual dataset.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '0', 'Create Virtual Point Cloud Dataset')
    if Tool.is_Okay():
        Tool.Set_Option('FILES', FILES)
        Tool.Set_Option('INPUT_FILE_LIST', INPUT_FILE_LIST)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('METHOD_PATHS', METHOD_PATHS)
        Tool.Set_Option('USE_HEADER', USE_HEADER)
        return Tool.Execute(Verbose)
    return False

def Get_Subset_from_Virtual_Point_Cloud(
    AOI_SHP=None,
    AOI_GRID=None,
    PC_OUT=None,
    FILENAME=None,
    FILEPATH=None,
    COPY_ATTR=None,
    ATTRIBUTE_LIST=None,
    CONSTRAIN_QUERY=None,
    ATTR_FIELD=None,
    VALUE_RANGE=None,
    FIELD_TILENAME=None,
    AOI_GRID_GRIDSYSTEM=None,
    AOI_XRANGE=None,
    AOI_YRANGE=None,
    AOI_ADD_OVERLAP=None,
    OVERLAP=None,
    FILENAME_TILE_INFO=None,
    ONE_PC_PER_POLYGON=None, Verbose=2):
    '''
    Get Subset from Virtual Point Cloud
    ----------
    [io_virtual.1]\n
    The tool allows one to retrieve a point cloud from a virtual point cloud dataset by applying the provided area-of-interest (AOI). The extent of the AOI can be provided either as polygon shapefile, grid or by coordinates. Optionally, an overlap can be added to the AOI and a spcvf tile info file can be outputted. The latter can be used to remove the overlap later.\n
    In case an overlap is used and the AOI is provided as polygon shapfile, only the bounding boxes of the polygons are used.\n
    With polygon shapefiles additional functionality is available:\n
    * in case one or more polygons are selected, only the selected polygons are used.\n
    * in case the shapefile contains several polygons and the 'One Point Cloud per Polygon' parameter is checked, a point cloud dataset is outputted for each polygon. In case the 'Tilename' attribute is provided, the output files are named by this attribute. Otherwise the output file names are build from the lower left coordinate of each tile.\n
    The derived datasets can be outputted either as point cloud list or written to an output directory. For the latter, you must provide a valid file path with the 'Optional Output Filepath' parameter.\n
    Optionally, the query can be constrained by providing an attribute field and a value range that must be met.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool.\n
    Arguments
    ----------
    - AOI_SHP [`optional input shapes`] : Shape. Shapefile describing the AOI.
    - AOI_GRID [`optional input grid`] : Grid. Grid describing the AOI.
    - PC_OUT [`output point cloud list`] : Point Cloud. The output point cloud(s)
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file
    - FILEPATH [`file path`] : Optional Output Filepath. The full path to which the output(s) should be written. Leave empty to output the datasets as point cloud list.
    - COPY_ATTR [`boolean`] : Copy existing Attributes. Default: 1 Copy attributes from input to output point cloud.
    - ATTRIBUTE_LIST [`text`] : Copy Attributes. Default: 1;2;3 Field numbers (starting from 1) of the attributes to copy, separated by semicolon; fields one to three (x;y;z) are mandatory.
    - CONSTRAIN_QUERY [`boolean`] : Constrain Query. Default: 0 Check this parameter to constrain the query by an attribute range.
    - ATTR_FIELD [`integer number`] : Attribute Field. Minimum: 1 Default: 1 The attribute field to use as constraint. Field numbers start with 1.
    - VALUE_RANGE [`value range`] : Value Range. Minimum and maximum of attribute range [].
    - FIELD_TILENAME [`table field`] : Tilename. Attribute used for naming the output file(s)
    - AOI_GRID_GRIDSYSTEM [`grid system`] : Grid system
    - AOI_XRANGE [`value range`] : X-Extent. Minimum and maximum x-coordinate of AOI.
    - AOI_YRANGE [`value range`] : Y-Extent. Minimum and maximum y-coordinate of AOI.
    - AOI_ADD_OVERLAP [`boolean`] : Add Overlap. Default: 0 Add overlap to AOI
    - OVERLAP [`floating point number`] : Overlap. Minimum: 0.000000 Default: 50.000000 Overlap [map units]
    - FILENAME_TILE_INFO [`file path`] : Optional Tile Info Filename. The full path and name of an optional spcvf tile info file. Such a file contains information about the bounding boxes without overlap and can be used to remove the overlap from the tiles later. Leave empty to not output such a file.
    - ONE_PC_PER_POLYGON [`boolean`] : One Point Cloud per Polygon. Default: 0 Write one point cloud dataset for each polygon

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '1', 'Get Subset from Virtual Point Cloud')
    if Tool.is_Okay():
        Tool.Set_Input ('AOI_SHP', AOI_SHP)
        Tool.Set_Input ('AOI_GRID', AOI_GRID)
        Tool.Set_Output('PC_OUT', PC_OUT)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('FILEPATH', FILEPATH)
        Tool.Set_Option('COPY_ATTR', COPY_ATTR)
        Tool.Set_Option('ATTRIBUTE_LIST', ATTRIBUTE_LIST)
        Tool.Set_Option('CONSTRAIN_QUERY', CONSTRAIN_QUERY)
        Tool.Set_Option('ATTR_FIELD', ATTR_FIELD)
        Tool.Set_Option('VALUE_RANGE', VALUE_RANGE)
        Tool.Set_Option('FIELD_TILENAME', FIELD_TILENAME)
        Tool.Set_Option('AOI_GRID_GRIDSYSTEM', AOI_GRID_GRIDSYSTEM)
        Tool.Set_Option('AOI_XRANGE', AOI_XRANGE)
        Tool.Set_Option('AOI_YRANGE', AOI_YRANGE)
        Tool.Set_Option('AOI_ADD_OVERLAP', AOI_ADD_OVERLAP)
        Tool.Set_Option('OVERLAP', OVERLAP)
        Tool.Set_Option('FILENAME_TILE_INFO', FILENAME_TILE_INFO)
        Tool.Set_Option('ONE_PC_PER_POLYGON', ONE_PC_PER_POLYGON)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_virtual_1(
    AOI_SHP=None,
    AOI_GRID=None,
    PC_OUT=None,
    FILENAME=None,
    FILEPATH=None,
    COPY_ATTR=None,
    ATTRIBUTE_LIST=None,
    CONSTRAIN_QUERY=None,
    ATTR_FIELD=None,
    VALUE_RANGE=None,
    FIELD_TILENAME=None,
    AOI_GRID_GRIDSYSTEM=None,
    AOI_XRANGE=None,
    AOI_YRANGE=None,
    AOI_ADD_OVERLAP=None,
    OVERLAP=None,
    FILENAME_TILE_INFO=None,
    ONE_PC_PER_POLYGON=None, Verbose=2):
    '''
    Get Subset from Virtual Point Cloud
    ----------
    [io_virtual.1]\n
    The tool allows one to retrieve a point cloud from a virtual point cloud dataset by applying the provided area-of-interest (AOI). The extent of the AOI can be provided either as polygon shapefile, grid or by coordinates. Optionally, an overlap can be added to the AOI and a spcvf tile info file can be outputted. The latter can be used to remove the overlap later.\n
    In case an overlap is used and the AOI is provided as polygon shapfile, only the bounding boxes of the polygons are used.\n
    With polygon shapefiles additional functionality is available:\n
    * in case one or more polygons are selected, only the selected polygons are used.\n
    * in case the shapefile contains several polygons and the 'One Point Cloud per Polygon' parameter is checked, a point cloud dataset is outputted for each polygon. In case the 'Tilename' attribute is provided, the output files are named by this attribute. Otherwise the output file names are build from the lower left coordinate of each tile.\n
    The derived datasets can be outputted either as point cloud list or written to an output directory. For the latter, you must provide a valid file path with the 'Optional Output Filepath' parameter.\n
    Optionally, the query can be constrained by providing an attribute field and a value range that must be met.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool.\n
    Arguments
    ----------
    - AOI_SHP [`optional input shapes`] : Shape. Shapefile describing the AOI.
    - AOI_GRID [`optional input grid`] : Grid. Grid describing the AOI.
    - PC_OUT [`output point cloud list`] : Point Cloud. The output point cloud(s)
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file
    - FILEPATH [`file path`] : Optional Output Filepath. The full path to which the output(s) should be written. Leave empty to output the datasets as point cloud list.
    - COPY_ATTR [`boolean`] : Copy existing Attributes. Default: 1 Copy attributes from input to output point cloud.
    - ATTRIBUTE_LIST [`text`] : Copy Attributes. Default: 1;2;3 Field numbers (starting from 1) of the attributes to copy, separated by semicolon; fields one to three (x;y;z) are mandatory.
    - CONSTRAIN_QUERY [`boolean`] : Constrain Query. Default: 0 Check this parameter to constrain the query by an attribute range.
    - ATTR_FIELD [`integer number`] : Attribute Field. Minimum: 1 Default: 1 The attribute field to use as constraint. Field numbers start with 1.
    - VALUE_RANGE [`value range`] : Value Range. Minimum and maximum of attribute range [].
    - FIELD_TILENAME [`table field`] : Tilename. Attribute used for naming the output file(s)
    - AOI_GRID_GRIDSYSTEM [`grid system`] : Grid system
    - AOI_XRANGE [`value range`] : X-Extent. Minimum and maximum x-coordinate of AOI.
    - AOI_YRANGE [`value range`] : Y-Extent. Minimum and maximum y-coordinate of AOI.
    - AOI_ADD_OVERLAP [`boolean`] : Add Overlap. Default: 0 Add overlap to AOI
    - OVERLAP [`floating point number`] : Overlap. Minimum: 0.000000 Default: 50.000000 Overlap [map units]
    - FILENAME_TILE_INFO [`file path`] : Optional Tile Info Filename. The full path and name of an optional spcvf tile info file. Such a file contains information about the bounding boxes without overlap and can be used to remove the overlap from the tiles later. Leave empty to not output such a file.
    - ONE_PC_PER_POLYGON [`boolean`] : One Point Cloud per Polygon. Default: 0 Write one point cloud dataset for each polygon

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '1', 'Get Subset from Virtual Point Cloud')
    if Tool.is_Okay():
        Tool.Set_Input ('AOI_SHP', AOI_SHP)
        Tool.Set_Input ('AOI_GRID', AOI_GRID)
        Tool.Set_Output('PC_OUT', PC_OUT)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('FILEPATH', FILEPATH)
        Tool.Set_Option('COPY_ATTR', COPY_ATTR)
        Tool.Set_Option('ATTRIBUTE_LIST', ATTRIBUTE_LIST)
        Tool.Set_Option('CONSTRAIN_QUERY', CONSTRAIN_QUERY)
        Tool.Set_Option('ATTR_FIELD', ATTR_FIELD)
        Tool.Set_Option('VALUE_RANGE', VALUE_RANGE)
        Tool.Set_Option('FIELD_TILENAME', FIELD_TILENAME)
        Tool.Set_Option('AOI_GRID_GRIDSYSTEM', AOI_GRID_GRIDSYSTEM)
        Tool.Set_Option('AOI_XRANGE', AOI_XRANGE)
        Tool.Set_Option('AOI_YRANGE', AOI_YRANGE)
        Tool.Set_Option('AOI_ADD_OVERLAP', AOI_ADD_OVERLAP)
        Tool.Set_Option('OVERLAP', OVERLAP)
        Tool.Set_Option('FILENAME_TILE_INFO', FILENAME_TILE_INFO)
        Tool.Set_Option('ONE_PC_PER_POLYGON', ONE_PC_PER_POLYGON)
        return Tool.Execute(Verbose)
    return False

def Create_Tileshape_from_Virtual_Point_Cloud(
    TILE_SHP=None,
    FILENAME=None, Verbose=2):
    '''
    Create Tileshape from Virtual Point Cloud
    ----------
    [io_virtual.2]\n
    The tool allows one to create a polygon shapefile with the bounding boxes of a virtual point cloud dataset. Additionally, the header information of the chosen virtual point cloud dataset is reported (since SPCVFDataset version 1.1).\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool.\n
    Arguments
    ----------
    - TILE_SHP [`output shapes`] : Tileshape. Polygon shapefile describing the bounding boxes of spcvf tiles.
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '2', 'Create Tileshape from Virtual Point Cloud')
    if Tool.is_Okay():
        Tool.Set_Output('TILE_SHP', TILE_SHP)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_virtual_2(
    TILE_SHP=None,
    FILENAME=None, Verbose=2):
    '''
    Create Tileshape from Virtual Point Cloud
    ----------
    [io_virtual.2]\n
    The tool allows one to create a polygon shapefile with the bounding boxes of a virtual point cloud dataset. Additionally, the header information of the chosen virtual point cloud dataset is reported (since SPCVFDataset version 1.1).\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool.\n
    Arguments
    ----------
    - TILE_SHP [`output shapes`] : Tileshape. Polygon shapefile describing the bounding boxes of spcvf tiles.
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '2', 'Create Tileshape from Virtual Point Cloud')
    if Tool.is_Okay():
        Tool.Set_Output('TILE_SHP', TILE_SHP)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def Get_Grid_from_Virtual_Point_Cloud(
    AOI_SHP=None,
    AOI_GRID=None,
    GRID_OUT=None,
    FILENAME=None,
    FILEPATH=None,
    ATTR_FIELD_GRID=None,
    CELL_SIZE=None,
    GRID_SYSTEM_FIT=None,
    METHOD=None,
    CONSTRAIN_QUERY=None,
    ATTR_FIELD=None,
    VALUE_RANGE=None,
    FIELD_TILENAME=None,
    AOI_GRID_GRIDSYSTEM=None,
    AOI_XRANGE=None,
    AOI_YRANGE=None,
    AOI_ADD_OVERLAP=None,
    OVERLAP=None, Verbose=2):
    '''
    Get Grid from Virtual Point Cloud
    ----------
    [io_virtual.4]\n
    The tool allows one to retrieve a grid from a virtual point cloud dataset by applying the provided area-of-interest (AOI). The extent of the AOI can be provided either as polygon shapefile, grid or by coordinates. Optionally, an overlap can be added to the AOI. In case an overlap is used and the AOI is provided as polygon shapfile, only the bounding boxes of the polygons are used.\n
    With polygon shapefiles additional functionality is available:\n
    * in case one or more polygons are selected, only the selected polygons are used.\n
    * in case the shapefile contains several polygons a grid dataset is outputted for each polygon. In case the 'Tilename' attribute is provided, the output files are named by this attribute. Otherwise the output file names are build from the lower left coordinate of each tile.\n
    The derived datasets can be outputted either as grid list or written to an output directory. For the latter, you must provide a valid file path with the 'Optional Output Filepath' parameter.\n
    Optionally, the query can be constrained by providing an attribute field and a value range that must be met.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool.\n
    Arguments
    ----------
    - AOI_SHP [`optional input shapes`] : Shape. Shapefile describing the AOI.
    - AOI_GRID [`optional input grid`] : Grid. Grid describing the AOI.
    - GRID_OUT [`output grid list`] : Grid. The output grid(s)
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file
    - FILEPATH [`file path`] : Optional Output Filepath. The full path to which the output(s) should be written. Leave empty to output the datasets as grid list.
    - ATTR_FIELD_GRID [`integer number`] : Attribute Field to Grid. Minimum: 3 Default: 1 The attribute field to grid. Field numbers start with 1, so elevation is attribute field 3.
    - CELL_SIZE [`floating point number`] : Cellsize. Minimum: 0.001000 Default: 1.000000 Cellsize of the output grid [map units]
    - GRID_SYSTEM_FIT [`choice`] : Grid System Fit. Available Choices: [0] nodes [1] cells Default: 1 Choose how to align the output grid system to the AOI
    - METHOD [`choice`] : Aggregation. Available Choices: [0] lowest [1] highest Default: 1 Choose how to aggregate the values
    - CONSTRAIN_QUERY [`boolean`] : Constrain Query. Default: 0 Check this parameter to constrain the query by an attribute range.
    - ATTR_FIELD [`integer number`] : Attribute Field. Minimum: 1 Default: 1 The attribute field to use as constraint. Field numbers start with 1.
    - VALUE_RANGE [`value range`] : Value Range. Minimum and maximum of attribute range [].
    - FIELD_TILENAME [`table field`] : Tilename. Attribute used for naming the output file(s)
    - AOI_GRID_GRIDSYSTEM [`grid system`] : Grid system
    - AOI_XRANGE [`value range`] : X-Extent. Minimum and maximum x-coordinate of AOI.
    - AOI_YRANGE [`value range`] : Y-Extent. Minimum and maximum y-coordinate of AOI.
    - AOI_ADD_OVERLAP [`boolean`] : Add Overlap. Default: 0 Add overlap to AOI
    - OVERLAP [`floating point number`] : Overlap. Minimum: 0.000000 Default: 50.000000 Overlap [map units]

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '4', 'Get Grid from Virtual Point Cloud')
    if Tool.is_Okay():
        Tool.Set_Input ('AOI_SHP', AOI_SHP)
        Tool.Set_Input ('AOI_GRID', AOI_GRID)
        Tool.Set_Output('GRID_OUT', GRID_OUT)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('FILEPATH', FILEPATH)
        Tool.Set_Option('ATTR_FIELD_GRID', ATTR_FIELD_GRID)
        Tool.Set_Option('CELL_SIZE', CELL_SIZE)
        Tool.Set_Option('GRID_SYSTEM_FIT', GRID_SYSTEM_FIT)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('CONSTRAIN_QUERY', CONSTRAIN_QUERY)
        Tool.Set_Option('ATTR_FIELD', ATTR_FIELD)
        Tool.Set_Option('VALUE_RANGE', VALUE_RANGE)
        Tool.Set_Option('FIELD_TILENAME', FIELD_TILENAME)
        Tool.Set_Option('AOI_GRID_GRIDSYSTEM', AOI_GRID_GRIDSYSTEM)
        Tool.Set_Option('AOI_XRANGE', AOI_XRANGE)
        Tool.Set_Option('AOI_YRANGE', AOI_YRANGE)
        Tool.Set_Option('AOI_ADD_OVERLAP', AOI_ADD_OVERLAP)
        Tool.Set_Option('OVERLAP', OVERLAP)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_virtual_4(
    AOI_SHP=None,
    AOI_GRID=None,
    GRID_OUT=None,
    FILENAME=None,
    FILEPATH=None,
    ATTR_FIELD_GRID=None,
    CELL_SIZE=None,
    GRID_SYSTEM_FIT=None,
    METHOD=None,
    CONSTRAIN_QUERY=None,
    ATTR_FIELD=None,
    VALUE_RANGE=None,
    FIELD_TILENAME=None,
    AOI_GRID_GRIDSYSTEM=None,
    AOI_XRANGE=None,
    AOI_YRANGE=None,
    AOI_ADD_OVERLAP=None,
    OVERLAP=None, Verbose=2):
    '''
    Get Grid from Virtual Point Cloud
    ----------
    [io_virtual.4]\n
    The tool allows one to retrieve a grid from a virtual point cloud dataset by applying the provided area-of-interest (AOI). The extent of the AOI can be provided either as polygon shapefile, grid or by coordinates. Optionally, an overlap can be added to the AOI. In case an overlap is used and the AOI is provided as polygon shapfile, only the bounding boxes of the polygons are used.\n
    With polygon shapefiles additional functionality is available:\n
    * in case one or more polygons are selected, only the selected polygons are used.\n
    * in case the shapefile contains several polygons a grid dataset is outputted for each polygon. In case the 'Tilename' attribute is provided, the output files are named by this attribute. Otherwise the output file names are build from the lower left coordinate of each tile.\n
    The derived datasets can be outputted either as grid list or written to an output directory. For the latter, you must provide a valid file path with the 'Optional Output Filepath' parameter.\n
    Optionally, the query can be constrained by providing an attribute field and a value range that must be met.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool.\n
    Arguments
    ----------
    - AOI_SHP [`optional input shapes`] : Shape. Shapefile describing the AOI.
    - AOI_GRID [`optional input grid`] : Grid. Grid describing the AOI.
    - GRID_OUT [`output grid list`] : Grid. The output grid(s)
    - FILENAME [`file path`] : Filename. The full path and name of the .spcvf file
    - FILEPATH [`file path`] : Optional Output Filepath. The full path to which the output(s) should be written. Leave empty to output the datasets as grid list.
    - ATTR_FIELD_GRID [`integer number`] : Attribute Field to Grid. Minimum: 3 Default: 1 The attribute field to grid. Field numbers start with 1, so elevation is attribute field 3.
    - CELL_SIZE [`floating point number`] : Cellsize. Minimum: 0.001000 Default: 1.000000 Cellsize of the output grid [map units]
    - GRID_SYSTEM_FIT [`choice`] : Grid System Fit. Available Choices: [0] nodes [1] cells Default: 1 Choose how to align the output grid system to the AOI
    - METHOD [`choice`] : Aggregation. Available Choices: [0] lowest [1] highest Default: 1 Choose how to aggregate the values
    - CONSTRAIN_QUERY [`boolean`] : Constrain Query. Default: 0 Check this parameter to constrain the query by an attribute range.
    - ATTR_FIELD [`integer number`] : Attribute Field. Minimum: 1 Default: 1 The attribute field to use as constraint. Field numbers start with 1.
    - VALUE_RANGE [`value range`] : Value Range. Minimum and maximum of attribute range [].
    - FIELD_TILENAME [`table field`] : Tilename. Attribute used for naming the output file(s)
    - AOI_GRID_GRIDSYSTEM [`grid system`] : Grid system
    - AOI_XRANGE [`value range`] : X-Extent. Minimum and maximum x-coordinate of AOI.
    - AOI_YRANGE [`value range`] : Y-Extent. Minimum and maximum y-coordinate of AOI.
    - AOI_ADD_OVERLAP [`boolean`] : Add Overlap. Default: 0 Add overlap to AOI
    - OVERLAP [`floating point number`] : Overlap. Minimum: 0.000000 Default: 50.000000 Overlap [map units]

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '4', 'Get Grid from Virtual Point Cloud')
    if Tool.is_Okay():
        Tool.Set_Input ('AOI_SHP', AOI_SHP)
        Tool.Set_Input ('AOI_GRID', AOI_GRID)
        Tool.Set_Output('GRID_OUT', GRID_OUT)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('FILEPATH', FILEPATH)
        Tool.Set_Option('ATTR_FIELD_GRID', ATTR_FIELD_GRID)
        Tool.Set_Option('CELL_SIZE', CELL_SIZE)
        Tool.Set_Option('GRID_SYSTEM_FIT', GRID_SYSTEM_FIT)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('CONSTRAIN_QUERY', CONSTRAIN_QUERY)
        Tool.Set_Option('ATTR_FIELD', ATTR_FIELD)
        Tool.Set_Option('VALUE_RANGE', VALUE_RANGE)
        Tool.Set_Option('FIELD_TILENAME', FIELD_TILENAME)
        Tool.Set_Option('AOI_GRID_GRIDSYSTEM', AOI_GRID_GRIDSYSTEM)
        Tool.Set_Option('AOI_XRANGE', AOI_XRANGE)
        Tool.Set_Option('AOI_YRANGE', AOI_YRANGE)
        Tool.Set_Option('AOI_ADD_OVERLAP', AOI_ADD_OVERLAP)
        Tool.Set_Option('OVERLAP', OVERLAP)
        return Tool.Execute(Verbose)
    return False

def Remove_Overlap_from_Virtual_Point_Cloud_Tiles(
    FILENAME=None,
    FILEPATH=None, Verbose=2):
    '''
    Remove Overlap from Virtual Point Cloud Tiles
    ----------
    [io_virtual.6]\n
    The tool allows one to remove the overlap from point cloud tiles created from a virtual point cloud dataset. The tiles must have been created with an overlap and a spcvf tile info file must have been outputted too. The latter describes the original bounding boxes of the tiles (i.e. without overlap) and is used by this tool to remove the overlap.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool. Point cloud tiles with an overlap are usually created from such an virtual point cloud dataset with the 'Get Subset from Virtual Point Cloud' tool.\n
    Arguments
    ----------
    - FILENAME [`file path`] : Tile Info File. The full path and name of the spcvf tile info file describing the point cloud tiles without overlap
    - FILEPATH [`file path`] : Output Filepath. The full path to which the point cloud tiles without overlap should be written.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '6', 'Remove Overlap from Virtual Point Cloud Tiles')
    if Tool.is_Okay():
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('FILEPATH', FILEPATH)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_virtual_6(
    FILENAME=None,
    FILEPATH=None, Verbose=2):
    '''
    Remove Overlap from Virtual Point Cloud Tiles
    ----------
    [io_virtual.6]\n
    The tool allows one to remove the overlap from point cloud tiles created from a virtual point cloud dataset. The tiles must have been created with an overlap and a spcvf tile info file must have been outputted too. The latter describes the original bounding boxes of the tiles (i.e. without overlap) and is used by this tool to remove the overlap.\n
    A virtual point cloud dataset is a simple XML format with the file extension .spcvf, which can be created with the 'Create Virtual Point Cloud Dataset' tool. Point cloud tiles with an overlap are usually created from such an virtual point cloud dataset with the 'Get Subset from Virtual Point Cloud' tool.\n
    Arguments
    ----------
    - FILENAME [`file path`] : Tile Info File. The full path and name of the spcvf tile info file describing the point cloud tiles without overlap
    - FILEPATH [`file path`] : Output Filepath. The full path to which the point cloud tiles without overlap should be written.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_virtual', '6', 'Remove Overlap from Virtual Point Cloud Tiles')
    if Tool.is_Okay():
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('FILEPATH', FILEPATH)
        return Tool.Execute(Verbose)
    return False




#________________________________________
def install(package):
    import subprocess, sys; subprocess.check_call([sys.executable, '-m', 'pip', 'install', package])

#________________________________________
def Create_Tarball(dir):
    import os, tarfile
    print('creating tarball \'{:s}\'...'.format(os.path.split(dir)[1]), end='', flush=True)
    tmp = tarfile.open('tmp.tar', 'w')
    tmp.add(dir, arcname=os.path.basename(dir))
    tmp.close()
    tmp = tarfile.open('tmp.tar', 'r')
    with tarfile.open(dir + '.tar.gz', 'w:gz') as tar:
        for entry in tmp.getmembers():
            if entry.isfile():
                entry.mode = 0o644
            else:
                entry.mode = 0o755
            tar.addfile(entry, tmp.extractfile(entry))
        tar.close()
    tmp.close()
    os.remove('tmp.tar')
    print('ready')

#________________________________________
def Create_ZipFile(dir):
    import os; from shutil import make_archive
    print('creating zipfile \'{:s}\'...'.format(os.path.split(dir)[1]), end='', flush=True)
    make_archive(dir, 'zip', None, dir)
    print('ready')

#________________________________________
if __name__ == '__main__':
    import sys, os
    if len(sys.argv) == 2 and os.path.exists(sys.argv[1]):
        Create_ZipFile(sys.argv[1])

    if len(sys.argv) >= 3 and os.path.exists(sys.argv[2]):
        if sys.argv[1] == 'tar':
            Create_Tarball(sys.argv[2])
        else:
            Create_ZipFile(sys.argv[2])



#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Simulation
- Name     : Air Flow Simulations
- ID       : sim_air_flow

Description
----------
Air Flow Simulations
'''

from PySAGA.helper import Tool_Wrapper

def Cold_Air_Flow(
    DEM=None,
    PRODUCTION=None,
    FRICTION=None,
    AIR=None,
    VELOCITY=None,
    PRODUCTION_DEFAULT=None,
    FRICTION_DEFAULT=None,
    RESET=None,
    TIME_STOP=None,
    TIME_UPDATE=None,
    EDGE=None,
    DELAY=None,
    T_AIR=None,
    T_AIR_COLD=None, Verbose=2):
    '''
    Cold Air Flow
    ----------
    [sim_air_flow.0]\n
    A simple cold air flow simulation.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - PRODUCTION [`optional input grid`] : Production. Rate of cold air production [m/h].
    - FRICTION [`optional input grid`] : Surface Friction Coefficient. Surface friction coefficient.
    - AIR [`output grid`] : Cold Air Height
    - VELOCITY [`output grid`] : Velocity
    - PRODUCTION_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 1.000000 default value if no grid has been selected
    - FRICTION_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 1.000000 default value if no grid has been selected
    - RESET [`boolean`] : Reset. Default: 1
    - TIME_STOP [`floating point number`] : Simulation Time [h]. Minimum: 0.000000 Default: 6.000000 Simulation time in hours.
    - TIME_UPDATE [`floating point number`] : Map Update Frequency [min]. Minimum: 0.000000 Default: 10.000000
    - EDGE [`choice`] : Edge. Available Choices: [0] closed [1] open Default: 1
    - DELAY [`floating point number`] : Time Step Adjustment. Minimum: 0.010000 Maximum: 1.000000 Default: 0.500000 Choosing a lower value will result in a better numerical precision but also in a longer calculation time.
    - T_AIR [`floating point number`] : Surrounding Air Temperature. Minimum: -273.150000 Default: 15.000000 Surrounding air temperature [degree Celsius].
    - T_AIR_COLD [`floating point number`] : Cold Air Temperature. Minimum: -273.150000 Default: 5.000000 Cold air temperature [degree Celsius].

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_air_flow', '0', 'Cold Air Flow')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('PRODUCTION', PRODUCTION)
        Tool.Set_Input ('FRICTION', FRICTION)
        Tool.Set_Output('AIR', AIR)
        Tool.Set_Output('VELOCITY', VELOCITY)
        Tool.Set_Option('PRODUCTION_DEFAULT', PRODUCTION_DEFAULT)
        Tool.Set_Option('FRICTION_DEFAULT', FRICTION_DEFAULT)
        Tool.Set_Option('RESET', RESET)
        Tool.Set_Option('TIME_STOP', TIME_STOP)
        Tool.Set_Option('TIME_UPDATE', TIME_UPDATE)
        Tool.Set_Option('EDGE', EDGE)
        Tool.Set_Option('DELAY', DELAY)
        Tool.Set_Option('T_AIR', T_AIR)
        Tool.Set_Option('T_AIR_COLD', T_AIR_COLD)
        return Tool.Execute(Verbose)
    return False

def run_tool_sim_air_flow_0(
    DEM=None,
    PRODUCTION=None,
    FRICTION=None,
    AIR=None,
    VELOCITY=None,
    PRODUCTION_DEFAULT=None,
    FRICTION_DEFAULT=None,
    RESET=None,
    TIME_STOP=None,
    TIME_UPDATE=None,
    EDGE=None,
    DELAY=None,
    T_AIR=None,
    T_AIR_COLD=None, Verbose=2):
    '''
    Cold Air Flow
    ----------
    [sim_air_flow.0]\n
    A simple cold air flow simulation.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - PRODUCTION [`optional input grid`] : Production. Rate of cold air production [m/h].
    - FRICTION [`optional input grid`] : Surface Friction Coefficient. Surface friction coefficient.
    - AIR [`output grid`] : Cold Air Height
    - VELOCITY [`output grid`] : Velocity
    - PRODUCTION_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 1.000000 default value if no grid has been selected
    - FRICTION_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 1.000000 default value if no grid has been selected
    - RESET [`boolean`] : Reset. Default: 1
    - TIME_STOP [`floating point number`] : Simulation Time [h]. Minimum: 0.000000 Default: 6.000000 Simulation time in hours.
    - TIME_UPDATE [`floating point number`] : Map Update Frequency [min]. Minimum: 0.000000 Default: 10.000000
    - EDGE [`choice`] : Edge. Available Choices: [0] closed [1] open Default: 1
    - DELAY [`floating point number`] : Time Step Adjustment. Minimum: 0.010000 Maximum: 1.000000 Default: 0.500000 Choosing a lower value will result in a better numerical precision but also in a longer calculation time.
    - T_AIR [`floating point number`] : Surrounding Air Temperature. Minimum: -273.150000 Default: 15.000000 Surrounding air temperature [degree Celsius].
    - T_AIR_COLD [`floating point number`] : Cold Air Temperature. Minimum: -273.150000 Default: 5.000000 Cold air temperature [degree Celsius].

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_air_flow', '0', 'Cold Air Flow')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('PRODUCTION', PRODUCTION)
        Tool.Set_Input ('FRICTION', FRICTION)
        Tool.Set_Output('AIR', AIR)
        Tool.Set_Output('VELOCITY', VELOCITY)
        Tool.Set_Option('PRODUCTION_DEFAULT', PRODUCTION_DEFAULT)
        Tool.Set_Option('FRICTION_DEFAULT', FRICTION_DEFAULT)
        Tool.Set_Option('RESET', RESET)
        Tool.Set_Option('TIME_STOP', TIME_STOP)
        Tool.Set_Option('TIME_UPDATE', TIME_UPDATE)
        Tool.Set_Option('EDGE', EDGE)
        Tool.Set_Option('DELAY', DELAY)
        Tool.Set_Option('T_AIR', T_AIR)
        Tool.Set_Option('T_AIR_COLD', T_AIR_COLD)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

#################################################################################
# MIT License

# Copyright (c) 2023 Olaf Conrad

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#################################################################################
# Purpose
#################################################################################

'''
The PySAGA.data.helper module provides a collection of functions supporting data access.
'''


#################################################################################
# Globals
#________________________________________________________________________________

import os; from PySAGA import saga_api


#################################################################################
#
# Downloads...
#________________________________________________________________________________

#________________________________________________________________________________
def Get_File(File, Local_Dir, Remote_Dir, Download_Retries = 4, Overwrite = False):
    '''
    If *Overwrite* is False this function checks if the requested 'File' exists in
    *Local_Dir* before it tries to download the file from *Remote_Dir* to *Local_Dir*.
    The function returns None if the file does not exist and the attempt to download
    it failed or the full file path to the local file otherwise.
    This function uses the *wget* Python module to download a requested file from
    internet. Any download will fail if *wget* is not installed.
    Installation can be done through pip:
        pip install wget
    '''

    Local_File = '{:s}/{:s}'.format(Local_Dir, File)
    if Overwrite == False and os.path.exists(Local_File):
        return Local_File

    if not os.path.exists(Local_Dir):
        os.makedirs(Local_Dir)

    Remote_File = '{:s}/{:s}'.format(Remote_Dir, File)

    Retry = Download_Retries
    if Retry < 0:
        Retry = 0
    Retry += 1

    try:
        import wget
    except:
        saga_api.SG_UI_Console_Print_StdErr('failed to import wget module! is it installed?')
        return None

    while Retry > 0:
        try:
            import wget
            print('\n')
            Local_File = wget.download(Remote_File, Local_Dir) # returns full path to downloaded file or 'None'
            if Local_File:
                saga_api.SG_UI_Console_Print_StdOut('')
                return Local_File
            Retry -= 1
        except: # remote file might not exist or internet connection is not available
            break

    saga_api.SG_UI_Console_Print_StdErr('failed to access remote file: ' + Remote_File)
    return None


#################################################################################
#
#________________________________________________________________________________



#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Terrain Analysis
- Name     : Channels
- ID       : ta_channels

Description
----------
Tools for (grid based) digital terrain analysis.
'''

from PySAGA.helper import Tool_Wrapper

def Channel_Network(
    ELEVATION=None,
    INIT_GRID=None,
    SINKROUTE=None,
    DIV_GRID=None,
    TRACE_WEIGHT=None,
    CHNLNTWRK=None,
    CHNLROUTE=None,
    SHAPES=None,
    INIT_METHOD=None,
    INIT_VALUE=None,
    DIV_CELLS=None,
    MINLEN=None, Verbose=2):
    '''
    Channel Network
    ----------
    [ta_channels.0]\n
    This tool derives a channel network based on gridded digital elevation data.\n
    Use the initiation options to determine under which conditions channels shall start.\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation. A grid that contains elevation data.
    - INIT_GRID [`input grid`] : Initiation Grid. Dependent on the chosen 'Initiation Type' and 'Initiation Threshold' the values of this grid control where a channel is initiated.
    - SINKROUTE [`optional input grid`] : Flow Direction. An optional grid that provides information about flow directions. 
Values between 1 to 8 force the flow of a cell to be given to one its adjacent neighbor cells (1->NE, 2->E, 3->SE, 4->S, 5->SW, 6->W, 7->NW, 8->N). In case of other values the algorithm will use its own routing scheme. 
This option is in particular useful to supply the algorithm with routes that lead the flow through closed depression. 
    - DIV_GRID [`optional input grid`] : Divergence. Tracing: Convergence
    - TRACE_WEIGHT [`optional input grid`] : Tracing: Weight. Tracing: Weight
    - CHNLNTWRK [`output grid`] : Channel Network. If a cell is part of a channel its value equals the channel order. Otherwise the cell is marked as no-data.
    - CHNLROUTE [`output grid`] : Channel Direction. If a cell is part of a channel then its value shows the flow direction of the channel (1->NE, 2->E, 3->SE, 4->S, 5->SW, 6->W, 7->NW, 8->N). Otherwise the cell is marked as no-data.
    - SHAPES [`output shapes`] : Channel Network. This shapes layer will contain the resulting channel network in vector format (lines).
    - INIT_METHOD [`choice`] : Initiation Type. Available Choices: [0] Less than [1] Equals [2] Greater than Default: 2 Options:
 - Less than
 - Equals
 - Greater than
Controls under which condition a channel is initiated.
    - INIT_VALUE [`floating point number`] : Initiation Threshold. Default: 0.000000 Dependent on the chosen 'Initiation Grid' and 'Initiation Type' this value controls under which condition a channel is initiated.
    - DIV_CELLS [`integer number`] : Tracing: Max. Divergence. Minimum: 1 Default: 5 Tracing: Stop after x cells with divergent flow
    - MINLEN [`integer number`] : Min. Segment Length. Default: 10 Minimum Segment Length (Cells)

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '0', 'Channel Network')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('INIT_GRID', INIT_GRID)
        Tool.Set_Input ('SINKROUTE', SINKROUTE)
        Tool.Set_Input ('DIV_GRID', DIV_GRID)
        Tool.Set_Input ('TRACE_WEIGHT', TRACE_WEIGHT)
        Tool.Set_Output('CHNLNTWRK', CHNLNTWRK)
        Tool.Set_Output('CHNLROUTE', CHNLROUTE)
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('INIT_METHOD', INIT_METHOD)
        Tool.Set_Option('INIT_VALUE', INIT_VALUE)
        Tool.Set_Option('DIV_CELLS', DIV_CELLS)
        Tool.Set_Option('MINLEN', MINLEN)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_0(
    ELEVATION=None,
    INIT_GRID=None,
    SINKROUTE=None,
    DIV_GRID=None,
    TRACE_WEIGHT=None,
    CHNLNTWRK=None,
    CHNLROUTE=None,
    SHAPES=None,
    INIT_METHOD=None,
    INIT_VALUE=None,
    DIV_CELLS=None,
    MINLEN=None, Verbose=2):
    '''
    Channel Network
    ----------
    [ta_channels.0]\n
    This tool derives a channel network based on gridded digital elevation data.\n
    Use the initiation options to determine under which conditions channels shall start.\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation. A grid that contains elevation data.
    - INIT_GRID [`input grid`] : Initiation Grid. Dependent on the chosen 'Initiation Type' and 'Initiation Threshold' the values of this grid control where a channel is initiated.
    - SINKROUTE [`optional input grid`] : Flow Direction. An optional grid that provides information about flow directions. 
Values between 1 to 8 force the flow of a cell to be given to one its adjacent neighbor cells (1->NE, 2->E, 3->SE, 4->S, 5->SW, 6->W, 7->NW, 8->N). In case of other values the algorithm will use its own routing scheme. 
This option is in particular useful to supply the algorithm with routes that lead the flow through closed depression. 
    - DIV_GRID [`optional input grid`] : Divergence. Tracing: Convergence
    - TRACE_WEIGHT [`optional input grid`] : Tracing: Weight. Tracing: Weight
    - CHNLNTWRK [`output grid`] : Channel Network. If a cell is part of a channel its value equals the channel order. Otherwise the cell is marked as no-data.
    - CHNLROUTE [`output grid`] : Channel Direction. If a cell is part of a channel then its value shows the flow direction of the channel (1->NE, 2->E, 3->SE, 4->S, 5->SW, 6->W, 7->NW, 8->N). Otherwise the cell is marked as no-data.
    - SHAPES [`output shapes`] : Channel Network. This shapes layer will contain the resulting channel network in vector format (lines).
    - INIT_METHOD [`choice`] : Initiation Type. Available Choices: [0] Less than [1] Equals [2] Greater than Default: 2 Options:
 - Less than
 - Equals
 - Greater than
Controls under which condition a channel is initiated.
    - INIT_VALUE [`floating point number`] : Initiation Threshold. Default: 0.000000 Dependent on the chosen 'Initiation Grid' and 'Initiation Type' this value controls under which condition a channel is initiated.
    - DIV_CELLS [`integer number`] : Tracing: Max. Divergence. Minimum: 1 Default: 5 Tracing: Stop after x cells with divergent flow
    - MINLEN [`integer number`] : Min. Segment Length. Default: 10 Minimum Segment Length (Cells)

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '0', 'Channel Network')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('INIT_GRID', INIT_GRID)
        Tool.Set_Input ('SINKROUTE', SINKROUTE)
        Tool.Set_Input ('DIV_GRID', DIV_GRID)
        Tool.Set_Input ('TRACE_WEIGHT', TRACE_WEIGHT)
        Tool.Set_Output('CHNLNTWRK', CHNLNTWRK)
        Tool.Set_Output('CHNLROUTE', CHNLROUTE)
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('INIT_METHOD', INIT_METHOD)
        Tool.Set_Option('INIT_VALUE', INIT_VALUE)
        Tool.Set_Option('DIV_CELLS', DIV_CELLS)
        Tool.Set_Option('MINLEN', MINLEN)
        return Tool.Execute(Verbose)
    return False

def Watershed_Basins(
    ELEVATION=None,
    CHANNELS=None,
    SINKROUTE=None,
    BASINS=None,
    MINSIZE=None, Verbose=2):
    '''
    Watershed Basins
    ----------
    [ta_channels.1]\n
    Watershed Basins\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation
    - CHANNELS [`input grid`] : Channel Network
    - SINKROUTE [`optional input grid`] : Sink Route
    - BASINS [`output grid`] : Watershed Basins
    - MINSIZE [`integer number`] : Min. Size. Default: 0 Minimum size of basin (cells)

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '1', 'Watershed Basins')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Input ('SINKROUTE', SINKROUTE)
        Tool.Set_Output('BASINS', BASINS)
        Tool.Set_Option('MINSIZE', MINSIZE)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_1(
    ELEVATION=None,
    CHANNELS=None,
    SINKROUTE=None,
    BASINS=None,
    MINSIZE=None, Verbose=2):
    '''
    Watershed Basins
    ----------
    [ta_channels.1]\n
    Watershed Basins\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation
    - CHANNELS [`input grid`] : Channel Network
    - SINKROUTE [`optional input grid`] : Sink Route
    - BASINS [`output grid`] : Watershed Basins
    - MINSIZE [`integer number`] : Min. Size. Default: 0 Minimum size of basin (cells)

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '1', 'Watershed Basins')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Input ('SINKROUTE', SINKROUTE)
        Tool.Set_Output('BASINS', BASINS)
        Tool.Set_Option('MINSIZE', MINSIZE)
        return Tool.Execute(Verbose)
    return False

def Watershed_Basins_Extended(
    DEM=None,
    CHANNELS=None,
    BASINS=None,
    SUBBASINS=None,
    V_BASINS=None,
    V_SUBBASINS=None,
    HEADS=None,
    MOUTHS=None,
    DISTANCE=None, Verbose=2):
    '''
    Watershed Basins (Extended)
    ----------
    [ta_channels.2]\n
    Extended watershed basin analysis.\n
    Arguments
    ----------
    - DEM [`input grid`] : DEM
    - CHANNELS [`input grid`] : Drainage Network
    - BASINS [`output grid`] : Basins
    - SUBBASINS [`output grid`] : Subbasins
    - V_BASINS [`output shapes`] : Basins
    - V_SUBBASINS [`output shapes`] : Subbasins
    - HEADS [`output shapes`] : River Heads
    - MOUTHS [`output shapes`] : River Mouths
    - DISTANCE [`boolean`] : Flow Distances. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '2', 'Watershed Basins (Extended)')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Output('BASINS', BASINS)
        Tool.Set_Output('SUBBASINS', SUBBASINS)
        Tool.Set_Output('V_BASINS', V_BASINS)
        Tool.Set_Output('V_SUBBASINS', V_SUBBASINS)
        Tool.Set_Output('HEADS', HEADS)
        Tool.Set_Output('MOUTHS', MOUTHS)
        Tool.Set_Option('DISTANCE', DISTANCE)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_2(
    DEM=None,
    CHANNELS=None,
    BASINS=None,
    SUBBASINS=None,
    V_BASINS=None,
    V_SUBBASINS=None,
    HEADS=None,
    MOUTHS=None,
    DISTANCE=None, Verbose=2):
    '''
    Watershed Basins (Extended)
    ----------
    [ta_channels.2]\n
    Extended watershed basin analysis.\n
    Arguments
    ----------
    - DEM [`input grid`] : DEM
    - CHANNELS [`input grid`] : Drainage Network
    - BASINS [`output grid`] : Basins
    - SUBBASINS [`output grid`] : Subbasins
    - V_BASINS [`output shapes`] : Basins
    - V_SUBBASINS [`output shapes`] : Subbasins
    - HEADS [`output shapes`] : River Heads
    - MOUTHS [`output shapes`] : River Mouths
    - DISTANCE [`boolean`] : Flow Distances. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '2', 'Watershed Basins (Extended)')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Output('BASINS', BASINS)
        Tool.Set_Output('SUBBASINS', SUBBASINS)
        Tool.Set_Output('V_BASINS', V_BASINS)
        Tool.Set_Output('V_SUBBASINS', V_SUBBASINS)
        Tool.Set_Output('HEADS', HEADS)
        Tool.Set_Output('MOUTHS', MOUTHS)
        Tool.Set_Option('DISTANCE', DISTANCE)
        return Tool.Execute(Verbose)
    return False

def Vertical_Distance_to_Channel_Network(
    ELEVATION=None,
    CHANNELS=None,
    DISTANCE=None,
    BASELEVEL=None,
    THRESHOLD=None,
    MAXITER=None,
    NOUNDERGROUND=None, Verbose=2):
    '''
    Vertical Distance to Channel Network
    ----------
    [ta_channels.3]\n
    This tool calculates the vertical distance to a channel network base level. The algorithm consists of two major steps:\n
    1. Interpolation of a channel network base level elevation\n
    2. Subtraction of this base level from the original elevations\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation. A grid that contains elevation data.
    - CHANNELS [`input grid`] : Channel Network. A grid providing information about the channel network. It is assumed that no-data cells are not part of the channel network. Vice versa all others cells are recognised as channel network members.
    - DISTANCE [`output grid`] : Vertical Distance to Channel Network. The resulting grid gives the altitude above the channel network in the same units as the elevation data.
    - BASELEVEL [`output grid`] : Channel Network Base Level. This optional grid output contains the interpolated channel network base level elevations.
    - THRESHOLD [`floating point number`] : Tension Threshold. Minimum: 0.000000 Default: 1.000000 Maximum change in elevation units (e.g. meter), iteration is stopped once maximum change reaches this threshold.
    - MAXITER [`integer number`] : Maximum Iterations. Minimum: 0 Default: 0 Maximum number of iterations, ignored if set to zero
    - NOUNDERGROUND [`boolean`] : Keep Base Level below Surface. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '3', 'Vertical Distance to Channel Network')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Output('DISTANCE', DISTANCE)
        Tool.Set_Output('BASELEVEL', BASELEVEL)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('MAXITER', MAXITER)
        Tool.Set_Option('NOUNDERGROUND', NOUNDERGROUND)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_3(
    ELEVATION=None,
    CHANNELS=None,
    DISTANCE=None,
    BASELEVEL=None,
    THRESHOLD=None,
    MAXITER=None,
    NOUNDERGROUND=None, Verbose=2):
    '''
    Vertical Distance to Channel Network
    ----------
    [ta_channels.3]\n
    This tool calculates the vertical distance to a channel network base level. The algorithm consists of two major steps:\n
    1. Interpolation of a channel network base level elevation\n
    2. Subtraction of this base level from the original elevations\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation. A grid that contains elevation data.
    - CHANNELS [`input grid`] : Channel Network. A grid providing information about the channel network. It is assumed that no-data cells are not part of the channel network. Vice versa all others cells are recognised as channel network members.
    - DISTANCE [`output grid`] : Vertical Distance to Channel Network. The resulting grid gives the altitude above the channel network in the same units as the elevation data.
    - BASELEVEL [`output grid`] : Channel Network Base Level. This optional grid output contains the interpolated channel network base level elevations.
    - THRESHOLD [`floating point number`] : Tension Threshold. Minimum: 0.000000 Default: 1.000000 Maximum change in elevation units (e.g. meter), iteration is stopped once maximum change reaches this threshold.
    - MAXITER [`integer number`] : Maximum Iterations. Minimum: 0 Default: 0 Maximum number of iterations, ignored if set to zero
    - NOUNDERGROUND [`boolean`] : Keep Base Level below Surface. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '3', 'Vertical Distance to Channel Network')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Output('DISTANCE', DISTANCE)
        Tool.Set_Output('BASELEVEL', BASELEVEL)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('MAXITER', MAXITER)
        Tool.Set_Option('NOUNDERGROUND', NOUNDERGROUND)
        return Tool.Execute(Verbose)
    return False

def Overland_Flow_Distance_to_Channel_Network(
    ELEVATION=None,
    CHANNELS=None,
    ROUTE=None,
    FIELDS=None,
    FLOW_K=None,
    FLOW_R=None,
    DISTANCE=None,
    DISTVERT=None,
    DISTHORZ=None,
    TIME=None,
    SDR=None,
    PASSES=None,
    METHOD=None,
    BOUNDARY=None,
    FLOW_B=None,
    FLOW_K_DEFAULT=None,
    FLOW_R_DEFAULT=None, Verbose=2):
    '''
    Overland Flow Distance to Channel Network
    ----------
    [ta_channels.4]\n
    This tool calculates overland flow distances to a channel network based on gridded digital elevation data and channel network information. The flow algorithm may be either Deterministic 8 (O'Callaghan & Mark 1984) or Multiple Flow Direction (Freeman 1991). Sediment Delivery Rates (SDR) according to Ali & De Boer (2010) can be computed optionally.\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation. A grid that contains elevation data.
    - CHANNELS [`input grid`] : Channel Network. A grid providing information about the channel network. It is assumed that no-data cells are not part of the channel network. Vice versa all others cells are recognised as channel network members.
    - ROUTE [`optional input grid`] : Preferred Routing. Downhill flow is bound to preferred routing cells, where these are not no-data. Helps to model e.g. small ditches, that are not well represented in the elevation data.
    - FIELDS [`optional input grid`] : Fields. If set, output is given about the number of fields a flow path visits downhill. For D8 only.
    - FLOW_K [`optional input grid`] : Manning-Strickler Coefficient. Manning-Strickler coefficient for flow travel time estimation (reciprocal of Manning's Roughness Coefficient)
    - FLOW_R [`optional input grid`] : Flow Depth. flow depth [m] for flow travel time estimation
    - DISTANCE [`output grid`] : Overland Flow Distance. The overland flow distance in map units. It is assumed that the (vertical) elevation data use the same units as the (horizontal) grid coordinates.
    - DISTVERT [`output grid`] : Vertical Overland Flow Distance. This is the vertical component of the overland flow
    - DISTHORZ [`output grid`] : Horizontal Overland Flow Distance. This is the horizontal component of the overland flow
    - TIME [`output grid`] : Flow Travel Time. flow travel time to channel expressed in hours based on Manning's Equation
    - SDR [`output grid`] : Sediment Yield Delivery Ratio. This is the horizontal component of the overland flow
    - PASSES [`output grid`] : Fields Visited. Number of fields a flow path visits downhill starting at a cell. For D8 only.
    - METHOD [`choice`] : Flow Algorithm. Available Choices: [0] D8 [1] MFD Default: 1 Choose a flow routing algorithm that shall be used for the overland flow distance calculation:
- D8
- MFD
    - BOUNDARY [`boolean`] : Boundary Cells. Default: 0 Take cells at the boundary of the DEM as channel.
    - FLOW_B [`floating point number`] : Beta. Minimum: 0.000000 Default: 1.000000 catchment specific parameter for sediment delivery ratio calculation
    - FLOW_K_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 20.000000 default value if no grid has been selected
    - FLOW_R_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 0.050000 default value if no grid has been selected

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '4', 'Overland Flow Distance to Channel Network')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Input ('ROUTE', ROUTE)
        Tool.Set_Input ('FIELDS', FIELDS)
        Tool.Set_Input ('FLOW_K', FLOW_K)
        Tool.Set_Input ('FLOW_R', FLOW_R)
        Tool.Set_Output('DISTANCE', DISTANCE)
        Tool.Set_Output('DISTVERT', DISTVERT)
        Tool.Set_Output('DISTHORZ', DISTHORZ)
        Tool.Set_Output('TIME', TIME)
        Tool.Set_Output('SDR', SDR)
        Tool.Set_Output('PASSES', PASSES)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('BOUNDARY', BOUNDARY)
        Tool.Set_Option('FLOW_B', FLOW_B)
        Tool.Set_Option('FLOW_K_DEFAULT', FLOW_K_DEFAULT)
        Tool.Set_Option('FLOW_R_DEFAULT', FLOW_R_DEFAULT)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_4(
    ELEVATION=None,
    CHANNELS=None,
    ROUTE=None,
    FIELDS=None,
    FLOW_K=None,
    FLOW_R=None,
    DISTANCE=None,
    DISTVERT=None,
    DISTHORZ=None,
    TIME=None,
    SDR=None,
    PASSES=None,
    METHOD=None,
    BOUNDARY=None,
    FLOW_B=None,
    FLOW_K_DEFAULT=None,
    FLOW_R_DEFAULT=None, Verbose=2):
    '''
    Overland Flow Distance to Channel Network
    ----------
    [ta_channels.4]\n
    This tool calculates overland flow distances to a channel network based on gridded digital elevation data and channel network information. The flow algorithm may be either Deterministic 8 (O'Callaghan & Mark 1984) or Multiple Flow Direction (Freeman 1991). Sediment Delivery Rates (SDR) according to Ali & De Boer (2010) can be computed optionally.\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation. A grid that contains elevation data.
    - CHANNELS [`input grid`] : Channel Network. A grid providing information about the channel network. It is assumed that no-data cells are not part of the channel network. Vice versa all others cells are recognised as channel network members.
    - ROUTE [`optional input grid`] : Preferred Routing. Downhill flow is bound to preferred routing cells, where these are not no-data. Helps to model e.g. small ditches, that are not well represented in the elevation data.
    - FIELDS [`optional input grid`] : Fields. If set, output is given about the number of fields a flow path visits downhill. For D8 only.
    - FLOW_K [`optional input grid`] : Manning-Strickler Coefficient. Manning-Strickler coefficient for flow travel time estimation (reciprocal of Manning's Roughness Coefficient)
    - FLOW_R [`optional input grid`] : Flow Depth. flow depth [m] for flow travel time estimation
    - DISTANCE [`output grid`] : Overland Flow Distance. The overland flow distance in map units. It is assumed that the (vertical) elevation data use the same units as the (horizontal) grid coordinates.
    - DISTVERT [`output grid`] : Vertical Overland Flow Distance. This is the vertical component of the overland flow
    - DISTHORZ [`output grid`] : Horizontal Overland Flow Distance. This is the horizontal component of the overland flow
    - TIME [`output grid`] : Flow Travel Time. flow travel time to channel expressed in hours based on Manning's Equation
    - SDR [`output grid`] : Sediment Yield Delivery Ratio. This is the horizontal component of the overland flow
    - PASSES [`output grid`] : Fields Visited. Number of fields a flow path visits downhill starting at a cell. For D8 only.
    - METHOD [`choice`] : Flow Algorithm. Available Choices: [0] D8 [1] MFD Default: 1 Choose a flow routing algorithm that shall be used for the overland flow distance calculation:
- D8
- MFD
    - BOUNDARY [`boolean`] : Boundary Cells. Default: 0 Take cells at the boundary of the DEM as channel.
    - FLOW_B [`floating point number`] : Beta. Minimum: 0.000000 Default: 1.000000 catchment specific parameter for sediment delivery ratio calculation
    - FLOW_K_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 20.000000 default value if no grid has been selected
    - FLOW_R_DEFAULT [`floating point number`] : Default. Minimum: 0.000000 Default: 0.050000 default value if no grid has been selected

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '4', 'Overland Flow Distance to Channel Network')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Input ('CHANNELS', CHANNELS)
        Tool.Set_Input ('ROUTE', ROUTE)
        Tool.Set_Input ('FIELDS', FIELDS)
        Tool.Set_Input ('FLOW_K', FLOW_K)
        Tool.Set_Input ('FLOW_R', FLOW_R)
        Tool.Set_Output('DISTANCE', DISTANCE)
        Tool.Set_Output('DISTVERT', DISTVERT)
        Tool.Set_Output('DISTHORZ', DISTHORZ)
        Tool.Set_Output('TIME', TIME)
        Tool.Set_Output('SDR', SDR)
        Tool.Set_Output('PASSES', PASSES)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('BOUNDARY', BOUNDARY)
        Tool.Set_Option('FLOW_B', FLOW_B)
        Tool.Set_Option('FLOW_K_DEFAULT', FLOW_K_DEFAULT)
        Tool.Set_Option('FLOW_R_DEFAULT', FLOW_R_DEFAULT)
        return Tool.Execute(Verbose)
    return False

def Channel_Network_and_Drainage_Basins(
    DEM=None,
    DIRECTION=None,
    CONNECTION=None,
    ORDER=None,
    BASIN=None,
    SEGMENTS=None,
    BASINS=None,
    NODES=None,
    THRESHOLD=None,
    SUBBASINS=None, Verbose=2):
    '''
    Channel Network and Drainage Basins
    ----------
    [ta_channels.5]\n
    Deterministic 8 based flow network analysis.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - DIRECTION [`output grid`] : Flow Direction
    - CONNECTION [`output grid`] : Flow Connectivity
    - ORDER [`output grid`] : Strahler Order
    - BASIN [`output grid`] : Drainage Basins
    - SEGMENTS [`output shapes`] : Channels
    - BASINS [`output shapes`] : Drainage Basins
    - NODES [`output shapes`] : Junctions
    - THRESHOLD [`integer number`] : Threshold. Minimum: 1 Default: 5 Strahler order to begin a channel.
    - SUBBASINS [`boolean`] : Subbasins. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '5', 'Channel Network and Drainage Basins')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Output('DIRECTION', DIRECTION)
        Tool.Set_Output('CONNECTION', CONNECTION)
        Tool.Set_Output('ORDER', ORDER)
        Tool.Set_Output('BASIN', BASIN)
        Tool.Set_Output('SEGMENTS', SEGMENTS)
        Tool.Set_Output('BASINS', BASINS)
        Tool.Set_Output('NODES', NODES)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('SUBBASINS', SUBBASINS)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_5(
    DEM=None,
    DIRECTION=None,
    CONNECTION=None,
    ORDER=None,
    BASIN=None,
    SEGMENTS=None,
    BASINS=None,
    NODES=None,
    THRESHOLD=None,
    SUBBASINS=None, Verbose=2):
    '''
    Channel Network and Drainage Basins
    ----------
    [ta_channels.5]\n
    Deterministic 8 based flow network analysis.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - DIRECTION [`output grid`] : Flow Direction
    - CONNECTION [`output grid`] : Flow Connectivity
    - ORDER [`output grid`] : Strahler Order
    - BASIN [`output grid`] : Drainage Basins
    - SEGMENTS [`output shapes`] : Channels
    - BASINS [`output shapes`] : Drainage Basins
    - NODES [`output shapes`] : Junctions
    - THRESHOLD [`integer number`] : Threshold. Minimum: 1 Default: 5 Strahler order to begin a channel.
    - SUBBASINS [`boolean`] : Subbasins. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '5', 'Channel Network and Drainage Basins')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Output('DIRECTION', DIRECTION)
        Tool.Set_Output('CONNECTION', CONNECTION)
        Tool.Set_Output('ORDER', ORDER)
        Tool.Set_Output('BASIN', BASIN)
        Tool.Set_Output('SEGMENTS', SEGMENTS)
        Tool.Set_Output('BASINS', BASINS)
        Tool.Set_Output('NODES', NODES)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('SUBBASINS', SUBBASINS)
        return Tool.Execute(Verbose)
    return False

def Strahler_Order(
    DEM=None,
    STRAHLER=None, Verbose=2):
    '''
    Strahler Order
    ----------
    [ta_channels.6]\n
    This tool allows one to calculate the Strahler stream order on basis of a DEM and the steepest descent (D8) algorithm.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation. The input elevation data set.
    - STRAHLER [`output grid`] : Strahler Order. The output data set with encoded Strahler stream order.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '6', 'Strahler Order')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Output('STRAHLER', STRAHLER)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_6(
    DEM=None,
    STRAHLER=None, Verbose=2):
    '''
    Strahler Order
    ----------
    [ta_channels.6]\n
    This tool allows one to calculate the Strahler stream order on basis of a DEM and the steepest descent (D8) algorithm.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation. The input elevation data set.
    - STRAHLER [`output grid`] : Strahler Order. The output data set with encoded Strahler stream order.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '6', 'Strahler Order')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Output('STRAHLER', STRAHLER)
        return Tool.Execute(Verbose)
    return False

def Valley_Depth(
    ELEVATION=None,
    VALLEY_DEPTH=None,
    RIDGE_LEVEL=None,
    THRESHOLD=None,
    MAXITER=None,
    NOUNDERGROUND=None,
    ORDER=None, Verbose=2):
    '''
    Valley Depth
    ----------
    [ta_channels.7]\n
    Valley depth is calculated as difference between the elevation and an interpolated ridge level. Ridge level interpolation uses the algorithm implemented in the 'Vertical Distance to Channel Network' tool. It performs the following steps:\n
    - Definition of ridge cells (using Strahler order on the inverted DEM).\n
    - Interpolation of the ridge level.\n
    - Subtraction of the original elevations from the ridge level.\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation
    - VALLEY_DEPTH [`output grid`] : Valley Depth
    - RIDGE_LEVEL [`output grid`] : Ridge Level
    - THRESHOLD [`floating point number`] : Tension Threshold. Minimum: 0.000000 Default: 1.000000 Maximum change in elevation units (e.g. meter), iteration is stopped once maximum change reaches this threshold.
    - MAXITER [`integer number`] : Maximum Iterations. Minimum: 0 Default: 0 Maximum number of iterations, ignored if set to zero
    - NOUNDERGROUND [`boolean`] : Keep Ridge Level above Surface. Default: 1
    - ORDER [`integer number`] : Ridge Detection Threshold. Minimum: 1 Maximum: 7 Default: 4

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '7', 'Valley Depth')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Output('VALLEY_DEPTH', VALLEY_DEPTH)
        Tool.Set_Output('RIDGE_LEVEL', RIDGE_LEVEL)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('MAXITER', MAXITER)
        Tool.Set_Option('NOUNDERGROUND', NOUNDERGROUND)
        Tool.Set_Option('ORDER', ORDER)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_channels_7(
    ELEVATION=None,
    VALLEY_DEPTH=None,
    RIDGE_LEVEL=None,
    THRESHOLD=None,
    MAXITER=None,
    NOUNDERGROUND=None,
    ORDER=None, Verbose=2):
    '''
    Valley Depth
    ----------
    [ta_channels.7]\n
    Valley depth is calculated as difference between the elevation and an interpolated ridge level. Ridge level interpolation uses the algorithm implemented in the 'Vertical Distance to Channel Network' tool. It performs the following steps:\n
    - Definition of ridge cells (using Strahler order on the inverted DEM).\n
    - Interpolation of the ridge level.\n
    - Subtraction of the original elevations from the ridge level.\n
    Arguments
    ----------
    - ELEVATION [`input grid`] : Elevation
    - VALLEY_DEPTH [`output grid`] : Valley Depth
    - RIDGE_LEVEL [`output grid`] : Ridge Level
    - THRESHOLD [`floating point number`] : Tension Threshold. Minimum: 0.000000 Default: 1.000000 Maximum change in elevation units (e.g. meter), iteration is stopped once maximum change reaches this threshold.
    - MAXITER [`integer number`] : Maximum Iterations. Minimum: 0 Default: 0 Maximum number of iterations, ignored if set to zero
    - NOUNDERGROUND [`boolean`] : Keep Ridge Level above Surface. Default: 1
    - ORDER [`integer number`] : Ridge Detection Threshold. Minimum: 1 Maximum: 7 Default: 4

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_channels', '7', 'Valley Depth')
    if Tool.is_Okay():
        Tool.Set_Input ('ELEVATION', ELEVATION)
        Tool.Set_Output('VALLEY_DEPTH', VALLEY_DEPTH)
        Tool.Set_Output('RIDGE_LEVEL', RIDGE_LEVEL)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('MAXITER', MAXITER)
        Tool.Set_Option('NOUNDERGROUND', NOUNDERGROUND)
        Tool.Set_Option('ORDER', ORDER)
        return Tool.Execute(Verbose)
    return False

